#------------------------------------------------------------------------------
#
#  Copyright (c) 2008, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#------------------------------------------------------------------------------

""" Defines the Boolean editor factory for all traits toolkit backends.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from __future__ import absolute_import

from traits.api import Dict, Str, Any

# CIRCULAR IMPORT FIXME: Importing from the source rather than traits.ui.api
# to avoid circular imports, as this EditorFactory will be part of
# traits.ui.api as well.

from ..view import View

from .text_editor import ToolkitEditorFactory as EditorFactory

#-------------------------------------------------------------------------------
#  Trait definitions:
#-------------------------------------------------------------------------------

# Mapping from user input text to Boolean values
mapping_trait = Dict( Str, Any, { 'True':  True,
                                  'true':  True,
                                  't':     True,
                                  'yes':   True,
                                  'y':     True,
                                  'False': False,
                                  'false': False,
                                  'f':     False,
                                  'no':    False,
                                  'n':     False,
                    } )

#-------------------------------------------------------------------------------
#  'ToolkitEditorFactory' class:
#-------------------------------------------------------------------------------

class ToolkitEditorFactory ( EditorFactory ):
    """ Editor factory for Boolean editors.
    """
    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Dictionary mapping user input to other values.
    # These definitions override definitions in the 'text_editor' version
    mapping = mapping_trait

    #---------------------------------------------------------------------------
    #  Traits view definition:
    #---------------------------------------------------------------------------

    traits_view = View()

    #---------------------------------------------------------------------------
    #  EditorFactory methods
    #---------------------------------------------------------------------------

    def _get_custom_editor_class(self):
        """ Returns the editor class to use for "custom" style views.
        Overridden to return the simple_editor_class (instead of the
        CustomEditor class for the text editor's factory, which this class
        inherits from).

        """
        return self.simple_editor_class


# Define the BooleanEditor class
BooleanEditor = ToolkitEditorFactory

#- EOF -----------------------------------------------------------------------



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/07/2004
#
#------------------------------------------------------------------------------

""" Exports the symbols defined by the traits.ui package.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from __future__ import absolute_import

from .basic_editor_factory import BasicEditorFactory

from .context_value import CV, CVFloat, CVInt, CVStr, CVType, ContextValue

from .editor import Editor

from .editor_factory import EditorFactory

from .editors.api import (ArrayEditor, BooleanEditor, ButtonEditor,
    CheckListEditor, CodeEditor, ColorEditor, CompoundEditor, CustomEditor,
    CSVListEditor,
    DNDEditor, StyledDateEditor, DateEditor, DefaultOverride, DirectoryEditor, DropEditor,
    EnumEditor, FileEditor, FontEditor, HTMLEditor, HistoryEditor, ImageEditor,
    ImageEnumEditor, InstanceEditor, KeyBindingEditor, ListEditor,
    ListStrEditor, NullEditor, PopupEditor, ProgressEditor, RGBColorEditor,
    RangeEditor, ScrubberEditor, SearchEditor, SetEditor, ShellEditor,
    TableEditor, TabularEditor, TextEditor, TimeEditor, TitleEditor, TreeEditor,
    TupleEditor, ValueEditor)

from .group import (Group, HFlow, HGroup, HSplit, Tabbed, VFlow, VFold, VGrid,
    VGroup, VSplit)

from .handler import Controller, Handler, ModelView, ViewHandler, default_handler

from .help import on_help_call

from .help_template import help_template

from .include import Include

from .item import (Custom, Heading, Item, Label, Readonly, Spring, UCustom,
    UItem, UReadonly, spring)

from .menu import (Action, ActionGroup, ApplyButton, CancelButton, CloseAction,
    HelpAction, HelpButton, LiveButtons, Menu, MenuBar, ModalButtons, NoButton,
    NoButtons, OKButton, OKCancelButtons, PyFaceAction, RedoAction,
    RevertAction, RevertButton, Separator, StandardMenuBar, ToolBar, UndoAction,
    UndoButton)

from .message import auto_close_message, error, message

from .table_column import (ExpressionColumn, ListColumn, NumericColumn,
    ObjectColumn, TableColumn)

from .table_filter import (EvalTableFilter, MenuTableFilter, RuleTableFilter,
    TableFilter)

from .theme import Theme, default_theme

from .toolkit import toolkit

from .toolkit_traits import ColorTrait, FontTrait, RGBColorTrait

from .tree_node import (ITreeNode, ITreeNodeAdapter, MultiTreeNode,
    ObjectTreeNode, TreeNode, TreeNodeObject)

from .ui import UI

from .ui_info import UIInfo

from .ui_traits import (ATheme, Border, HasBorder, HasMargin, Image, Margin,
    StatusItem)

from .undo import (AbstractUndoItem, ListUndoItem, UndoHistory,
    UndoHistoryUndoItem, UndoItem)

from .view import View

from .view_element import ViewElement, ViewSubElement

from . import view_elements

_constants  = toolkit().constants()
WindowColor = _constants.get( 'WindowColor', 0xFFFFFF )


def raise_to_debug() :
    """ When we would otherwise silently swallow an exception, call this instead
    to allow people to set the TRAITS_DEBUG environment variable and get the
    exception.
    """
    import os
    if os.getenv('TRAITS_DEBUG') is not None : raise



""" Map editor

"""
# Enthought library imports.
from pyface.api import GUI, ImageResource, ConfirmationDialog, FileDialog, \
    ImageResource, YES, OK, CANCEL
from pyface.action.api import Action, ActionItem, Separator, Group
from pyface.tasks.api import Task, TaskWindow, TaskLayout, PaneItem, IEditor, \
    IEditorAreaPane, EditorAreaPane, Editor, DockPane, HSplitter, VSplitter
from pyface.tasks.action.api import DockPaneToggleGroup, SMenuBar, \
    SMenu, SToolBar, TaskAction, TaskToggleGroup, EditorAction, SchemaAddition
from traits.api import on_trait_change, Property, Instance, Any, Event, Int

from omnivore.framework.actions import *
from map_editor import MapEditor
from preferences import MapEditPreferences
from commands import *
from omnivore.tasks.hex_edit.task import HexEditTask
from omnivore.tasks.hex_edit.actions import *
import pane_layout
import omnivore.utils.wx.fonts as fonts
from omnivore.utils.binutil import known_segment_parsers
import omnivore.utils.colors as colors
from omnivore.framework.toolbar import get_toolbar_group


class MapEditTask(HexEditTask):
    """ Tile-based map editor
    """

    new_file_text = "Map File"

    #### Task interface #######################################################

    id = pane_layout.task_id_with_pane_layout
    name = 'Map Editor'
    
    preferences_helper = MapEditPreferences
    
    #### Menu events ##########################################################
    
    fonts_changed = Event
    
    segments_changed = Event

    ###########################################################################
    # 'Task' interface.
    ###########################################################################

    def _default_layout_default(self):
        return pane_layout.pane_layout()

    def create_dock_panes(self):
        return pane_layout.pane_create()

    def _extra_actions_default(self):
        segment_menu = self.create_menu("Menu", "Segments", "SegmentParserGroup", "SegmentGroup")
        tiles_menu = self.create_menu("Menu", "Tiles", "TileModifyGroup")
        actions = [
            # Menubar additions
            SchemaAddition(factory=lambda: segment_menu,
                           path='MenuBar',
                           after="Edit",
                           ),
            SchemaAddition(factory=lambda: tiles_menu,
                           path='MenuBar',
                           after="Edit",
                           ),
            ]
        return actions

    def _tool_bars_default(self):
        toolbars = []
        toolbars.append(get_toolbar_group("Modes", MapEditor.valid_mouse_modes))
        toolbars.extend(HexEditTask._tool_bars_default(self))
        return toolbars

    ###########################################################################
    # 'FrameworkTask' interface.
    ###########################################################################

    def get_editor(self, guess=None):
        """ Opens a new empty window
        """
        editor = MapEditor()
        return editor
    
    def get_actions_Menu_View_ViewConfigGroup(self):
        return self.get_common_ViewConfigGroup()

    ###
    @classmethod
    def can_edit(cls, document):
        return document.metadata.mime == "application/octet-stream" or document.segments
    
    @classmethod
    def get_match_score(cls, document):
        """Return a number based on how good of a match this task is to the
        incoming Document.
        
        0 = generic match
        ...
        10 = absolute match
        """
        return 0



#-------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   09/22/2005
#
#-------------------------------------------------------------------------------

""" Converts a wx.KeyEvent to a standardized "name".
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

#-------------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------------

# Mapping from wxPython special key names to Enable key names
key_map = {
    wx.WXK_BACK:      'Backspace',
    wx.WXK_TAB:       'Tab',
    wx.WXK_RETURN:    'Enter',
    wx.WXK_ESCAPE:    'Esc',
    wx.WXK_DELETE:    'Delete',
    wx.WXK_START:     'Start',
    wx.WXK_LBUTTON:   'Left Button',
    wx.WXK_RBUTTON:   'Right Button',
    wx.WXK_CANCEL:    'Cancel',
    wx.WXK_MBUTTON:   'Middle Button',
    wx.WXK_CLEAR:     'Clear',
    wx.WXK_SHIFT:     'Shift',
    wx.WXK_CONTROL:   'Control',
    wx.WXK_MENU:      'Menu',
    wx.WXK_PAUSE:     'Pause',
    wx.WXK_CAPITAL:   'Capital',
    wx.WXK_PRIOR:     'Page Up',
    wx.WXK_NEXT:      'Page Down',
    wx.WXK_END:       'End',
    wx.WXK_HOME:      'Home',
    wx.WXK_LEFT:      'Left',
    wx.WXK_UP:        'Up',
    wx.WXK_RIGHT:     'Right',
    wx.WXK_DOWN:      'Down',
    wx.WXK_SELECT:    'Select',
    wx.WXK_PRINT:     'Print',
    wx.WXK_EXECUTE:   'Execute',
    wx.WXK_SNAPSHOT:  'Snapshot',
    wx.WXK_INSERT:    'Insert',
    wx.WXK_HELP:      'Help',
    wx.WXK_NUMPAD0:   'Numpad 0',
    wx.WXK_NUMPAD1:   'Numpad 1',
    wx.WXK_NUMPAD2:   'Numpad 2',
    wx.WXK_NUMPAD3:   'Numpad 3',
    wx.WXK_NUMPAD4:   'Numpad 4',
    wx.WXK_NUMPAD5:   'Numpad 5',
    wx.WXK_NUMPAD6:   'Numpad 6',
    wx.WXK_NUMPAD7:   'Numpad 7',
    wx.WXK_NUMPAD8:   'Numpad 8',
    wx.WXK_NUMPAD9:   'Numpad 9',
    wx.WXK_MULTIPLY:  'Multiply',
    wx.WXK_ADD:       'Add',
    wx.WXK_SEPARATOR: 'Separator',
    wx.WXK_SUBTRACT:  'Subtract',
    wx.WXK_DECIMAL:   'Decimal',
    wx.WXK_DIVIDE:    'Divide',
    wx.WXK_F1:        'F1',
    wx.WXK_F2:        'F2',
    wx.WXK_F3:        'F3',
    wx.WXK_F4:        'F4',
    wx.WXK_F5:        'F5',
    wx.WXK_F6:        'F6',
    wx.WXK_F7:        'F7',
    wx.WXK_F8:        'F8',
    wx.WXK_F9:        'F9',
    wx.WXK_F10:       'F10',
    wx.WXK_F11:       'F11',
    wx.WXK_F12:       'F12',
    wx.WXK_F13:       'F13',
    wx.WXK_F14:       'F14',
    wx.WXK_F15:       'F15',
    wx.WXK_F16:       'F16',
    wx.WXK_F17:       'F17',
    wx.WXK_F18:       'F18',
    wx.WXK_F19:       'F19',
    wx.WXK_F20:       'F20',
    wx.WXK_F21:       'F21',
    wx.WXK_F22:       'F22',
    wx.WXK_F23:       'F23',
    wx.WXK_F24:       'F24',
    wx.WXK_NUMLOCK:   'Num Lock',
    wx.WXK_SCROLL:    'Scroll Lock'
}

#-------------------------------------------------------------------------------
#  Converts a keystroke event into a corresponding key name:
#-------------------------------------------------------------------------------

def key_event_to_name ( event ):
    """ Converts a keystroke event into a corresponding key name.
    """
    key_code = event.GetKeyCode()
    if event.ControlDown() and (1 <= key_code <= 26):
        key = chr( key_code + 96 )
    else:
        key = key_map.get( key_code )
        if key is None:
            try:
                key = chr( key_code )
            except:
                # Handle the case of strange keyboard codes (such as the Apple
                # keyboard 'apple' key):
                key = 'unknown'

    name = ''
    if event.AltDown():
        name = 'Alt'

    if event.ControlDown():
        name += '-Ctrl'

    if event.ShiftDown() and ((name != '') or (len( key ) > 1)):
        name += '-Shift'

    if key == ' ':
        key = 'Space'

    if len( name ) > 0:
        key = key.lower()

    name += ('-' + key)

    if name[:1] == '-':
        return name[1:]
    return name




#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" The implementation of a dialog that displays a message. """


# Convenience functions.
def information(parent, message, title='Information'):
    """ Convenience function to show an information message dialog. """

    dialog = MessageDialog(
        parent=parent, message=message, title=title, severity='information'
    )
    dialog.open()

    return

def warning(parent, message, title='Warning'):
    """ Convenience function to show a warning message dialog. """

    dialog = MessageDialog(
        parent=parent, message=message, title=title, severity='warning'
    )
    dialog.open()

    return

def error(parent, message, title='Error'):
    """ Convenience function to show an error message dialog. """

    dialog = MessageDialog(
        parent=parent, message=message, title=title, severity='error'
    )
    dialog.open()

    return


# Import the toolkit specific version.
from toolkit import toolkit_object
MessageDialog = toolkit_object('message_dialog:MessageDialog')

#### EOF ######################################################################



#!/usr/bin/env python

import sys
import traceback
import wx


class UndoHistoryPanel(wx.Panel):
    def __init__(self, parent, task, **kwargs):
        self.task = task
        wx.Panel.__init__(self, parent, wx.ID_ANY, **kwargs)
        
        # Mac/Win needs this, otherwise background color is black
        attr = self.GetDefaultAttributes()
        self.SetBackgroundColour(attr.colBg)

        self.sizer = wx.BoxSizer(wx.VERTICAL)

        self.history = wx.ListBox(self, size=(100, -1))

        self.sizer.Add(self.history, 1, wx.EXPAND)

        self.SetSizer(self.sizer)
        self.sizer.Layout()
        self.Fit()
    
    def set_task(self, task):
        self.task = task

    def update_history(self):
        project = self.task.active_editor
        summary = project.document.undo_stack.history_list()
        self.history.Set(summary)
        index = project.document.undo_stack.insert_index
        if index > 0:
            self.history.SetSelection(index - 1)



import wx

from traits.api import Instance
from traitsui.wx.editor import Editor
from pyface.ui.wx.progress_dialog import ProgressDialog

class SimpleEditor(Editor):
    """
    Show a progress bar with all the optional goodies

    """

    progress = Instance(ProgressDialog)

    #-- Editor interface ------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.control = self.create_control( parent )
        self.set_tooltip()

    def create_control (self, parent):
        """
        Finishes initializing the editor by creating the underlying widget.
        """

        self.progress = ProgressDialog( title=self.factory.title,
                                        message=self.factory.message,
                                        min=self.factory.min,
                                        max=self.factory.max,
                                        can_cancel=self.factory.can_cancel,
                                        show_time=self.factory.show_time,
                                        show_percent=self.factory.show_percent)

        panel = wx.Panel(parent, -1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)
        panel.SetBackgroundColour(wx.NullColor)

        self.progress.dialog_size = wx.Size()

        # The 'guts' of the dialog.
        self.progress._create_message(panel, sizer)
        self.progress._create_gauge(panel, sizer)
        self.progress._create_percent(panel, sizer)
        self.progress._create_timer(panel, sizer)
        self.progress._create_buttons(panel, sizer)

        panel.SetClientSize(self.progress.dialog_size)

        panel.CentreOnParent()

        self.control = panel
        return self.control


    def update_editor ( self ):
        """
        Updates the editor when the object trait changes externally to the
        editor.
        """
        if self.value:
            self.progress.update(self.value)
        return



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/25/2004
#
#------------------------------------------------------------------------------

""" Defines various helper functions that are useful for creating Traits-based
    user interfaces.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from __future__ import absolute_import

from string import uppercase, lowercase

from traits.api import BaseTraitHandler, CTrait, Enum, TraitError

from .ui_traits import SequenceTypes

#-------------------------------------------------------------------------------
#  Trait definitions:
#-------------------------------------------------------------------------------

# Layout orientation for a control and its associated editor
Orientation = Enum( 'horizontal', 'vertical' )

# Docking drag bar style:
DockStyle = Enum( 'horizontal', 'vertical', 'tab', 'fixed' )

#----------------------------------------------------------------------------
#  Return a 'user-friendly' name for a specified trait:
#----------------------------------------------------------------------------

def user_name_for ( name ):
    """ Returns a "user-friendly" name for a specified trait.
    """
    name       = name.replace( '_', ' ' )
    name       = name[:1].upper() + name[1:]
    result     = ''
    last_lower = 0
    for c in name:
        if (c in uppercase) and last_lower:
           result += ' '
        last_lower = (c in lowercase)
        result    += c
    return result

#-------------------------------------------------------------------------------
#  Format a number with embedded commas:
#-------------------------------------------------------------------------------

def commatize ( value ):
    """ Formats a specified value as an integer string with embedded commas.
        For example: commatize( 12345 ) returns "12,345".
    """
    s = str( abs( value ) )
    s = s.rjust( ((len( s ) + 2) / 3) * 3 )
    result = ','.join( [ s[ i: i+3 ] for i in range( 0, len(s), 3 ) ] ).lstrip()
    if value >= 0:
        return result

    return '-' + result

#-------------------------------------------------------------------------------
#  Recomputes the mappings for a new set of enumeration values:
#-------------------------------------------------------------------------------

def enum_values_changed ( values, strfunc=unicode ):
    """ Recomputes the mappings for a new set of enumeration values.
    """

    if isinstance( values, dict ):
        data = [ ( strfunc( v ), n ) for n, v in values.items() ]
        if len( data ) > 0:
            data.sort( lambda x, y: cmp( x[0], y[0] ) )
            col = data[0][0].find( ':' ) + 1
            if col > 0:
                data = [ ( n[ col: ], v ) for n, v in data ]
    elif not isinstance( values, SequenceTypes ):
        handler = values
        if isinstance( handler, CTrait ):
            handler = handler.handler
        if not isinstance( handler, BaseTraitHandler ):
            raise TraitError, "Invalid value for 'values' specified"
        if handler.is_mapped:
            data = [ ( strfunc( n ), n ) for n in handler.map.keys() ]
            data.sort( lambda x, y: cmp( x[0], y[0] ) )
        else:
            data = [ ( strfunc( v ), v ) for v in handler.values ]
    else:
        data = [ ( strfunc( v ), v ) for v in values ]

    names           = [ x[0] for x in data ]
    mapping         = {}
    inverse_mapping = {}
    for name, value in data:
        mapping[ name ] = value
        inverse_mapping[ value ] = name

    return ( names, mapping, inverse_mapping )



import sys
import wx

from omnivore.utils.wx.bytegrid import ByteGridTable, ByteGrid, HexTextCtrl, HexCellEditor
from omnivore.utils.binutil import DefaultSegment

from omnivore.third_party.asm6502 import assemble_text, AssemblyError

from omnivore.framework.actions import *
from actions import *
from commands import MiniAssemblerCommand

import logging
log = logging.getLogger(__name__)


class DisassemblyTable(ByteGridTable):
    column_labels = ["Bytes", "Disassembly", "Comment"]
    column_sizes = [8, 12, 20]
    
    def __init__(self):
        ByteGridTable.__init__(self)
        self.lines = []
        self._rows = 0
        self.index_to_row = []
        self.start_addr = 0
        self.next_row = -1
        self.chunk_size = 256
        self.disassembler = None

    def set_editor(self, editor):
        self.editor = editor
        self.segment = segment = self.editor.segment
        self.lines = []
        self.index_to_row = []
        self._rows = 0
        self.disassembler = editor.disassembler(segment.data, segment.start_addr, -segment.start_addr, editor.task.hex_grid_lower_case, editor.task.assembly_lower_case)
        self.disassembler.set_pc(segment.data, segment.start_addr)
        self.next_row = 0
        self.start_addr = segment.start_addr
        self.disassemble_next()
    
    def restart_disassembly(self, index):
        try:
            next_row = self.index_to_row[index]
        except IndexError:
            # requesting an index that has yet to be disassembled, so it will
            # get there when it gets there! Be patient!
            return
        
        # don't reset starting point if the requested index is already in the
        # region to be rebuilt
        if self.next_row < 0 or next_row < self.next_row:
            self.next_row = next_row
            pc = self.get_pc(self.next_row)
            self.disassembler.set_pc(self.segment.data, pc)
    
    def disassemble_next(self):
        if self.next_row < 0:
            return
        lines = []
        start_row = self.next_row
        try:
            for i in range(self.chunk_size):
                (addr, bytes, opstr, comment) = self.disassembler.get_instruction()
                count = len(bytes)
                data = (addr, bytes, opstr, comment, count)
                lines.append(data)
        except StopIteration:
            pass
        if lines:
            n = len(lines)
            self.lines[start_row:start_row+n] = lines
            start_index, index_to_row = self.get_index_to_row(lines, start_row)
            self.index_to_row[start_index:start_index+len(index_to_row)] = index_to_row
            self.next_row += n
        else:
            self.lines[self.next_row:] = []
            self.next_row = -1
    
    def get_index_to_row(self, lines, start_row):
        row = start_row
        index = lines[0][0] - self.start_addr
        counts = [d[4] for d in lines]
        index_to_row = []
        for c in counts:
            index_to_row.extend([row] * c)
            row += 1
        return index, index_to_row
        
    def set_grid_cell_attr(self, grid, col, attr):
        ByteGridTable.set_grid_cell_attr(self, grid, col, attr)
        if col == 1:
            attr.SetReadOnly(False)
        else:
            attr.SetReadOnly(True)
    
    def get_index_range(self, r, c):
        line = self.lines[r]
        index = line[0] - self.start_addr
        return index, index + line[4]
    
    def is_index_valid(self, index):
        return index < len(self.segment)
    
    def get_row_col(self, index):
        try:
            row = self.index_to_row[index]
        except:
            row = self.index_to_row[-1]
        return row, 0

    def get_next_cursor_pos(self, row, col):
        col += 1
        if col >= self._cols:
            if row < self._rows - 1:
                row += 1
                col = 1
            else:
                col = self._cols - 1
        return (row, col)

    def get_next_editable_pos(self, row, col):
        if col < 1:
            col = 1
        else:
            col = 1
            row += 1
        return (row, col)
   
    def get_prev_cursor_pos(self, row, col):
        col -= 1
        if col < 1:
            if row > 0:
                row -= 1
                col = self._cols - 1
            else:
                col = 1
        return (row, col)
    
    def get_pc(self, row):
        return self.lines[row][0]

    def get_value_style_lower(self, row, col):
        line = self.lines[row]
        index = line[0] - self.start_addr
        style = 0
        for i in range(line[4]):
            style |= self.segment.style[index + i]
        if col == 0:
            return " ".join("%02x" % i for i in line[1]), style
        return str(line[col + 1]), style
    
    def get_value_style_upper(self, row, col):
        line = self.lines[row]
        index = line[0] - self.start_addr
        style = 0
        for i in range(line[4]):
            style |= self.segment.style[index + i]
        if col == 0:
            return " ".join("%02X" % i for i in line[1]), style
        return str(line[col + 1]), style
    
    def GetRowLabelValue(self, row):
        if self.lines:
            addr = self.get_pc(row)
            if self.get_value_style == self.get_value_style_lower:
                return "%04x" % addr
            return "%04X" % addr
        return "0000"

    def ResetViewProcessArgs(self, grid, editor, *args):
        if editor is not None:
            self.set_editor(editor)
        else:
            self._rows = len(self.lines)


class AssemblerTextCtrl(HexTextCtrl):
    def setMode(self, mode):
        self.mode='6502'
        self.SetMaxLength(0)
        self.autoadvance=0
        self.userpressed=False

class AssemblerEditor(HexCellEditor):
    def Create(self, parent, id, evtHandler):
        """
        Called to create the control, which must derive from wx.Control.
        *Must Override*
        """
        self._tc = AssemblerTextCtrl(parent, id, self.parentgrid)
        self.SetControl(self._tc)

        if evtHandler:
            self._tc.PushEventHandler(evtHandler)


class DisassemblyPanel(ByteGrid):
    """
    View for editing in hexidecimal notation.
    """

    def __init__(self, parent, task, **kwargs):
        """Create the HexEdit viewer
        """
        table = DisassemblyTable()
        ByteGrid.__init__(self, parent, task, table, **kwargs)
        
        # During idle-time disassembly, an index may not yet be visible.  The
        # value is saved here so the view can be scrolled there once it does
        # get disassembled.
        self.pending_index = -1
    
    def get_default_cell_editor(self):
        return AssemblerEditor(self)

    def perform_idle(self):
        if self.table.next_row >= 0:
            self.table.disassemble_next()
            self.table.ResetView(self, None)
            if self.pending_index >= 0:
                self.goto_index(self.pending_index)
    
    def restart_disassembly(self, index):
        self.table.restart_disassembly(index)

    def goto_index(self, index):
        try:
            row = self.table.index_to_row[index]
            self.pending_index = -1
        except IndexError:
            self.pending_index = index
        else:
            row, col = self.table.get_row_col(index)
            self.SetGridCursor(row, col)
            self.MakeCellVisible(row,col)
        
    def change_value(self, row, col, text):
        """Called after editor has provided a new value for a cell.
        
        Can use this to override the default handler.  Return True if the grid
        should be updated, or False if the value is invalid or the grid will
        be updated some other way.
        """
        try:
            pc = self.table.get_pc(row)
            cmd = text.upper()
            bytes = assemble_text(cmd, pc)
            start, _ = self.table.get_index_range(row, col)
            end = start + len(bytes)
            cmd = MiniAssemblerCommand(self.table.segment, start, end, bytes, cmd)
            self.task.active_editor.process_command(cmd)
            return True
        except AssemblyError, e:
            self.task.window.error(unicode(e))
            self.SetFocus()  # OS X quirk: return focus to the grid so the user can keep typing
        return False
    
    def search(self, search_text, match_case=False):
        lines = self.table.lines
        s = self.table.start_addr
        if not match_case:
            search_text = search_text.lower()
            matches = [(t[0] - s, t[0] - s + len(t[1])) for t in lines if search_text in t[3].lower()]
        else:
            matches = [(t[0] - s, t[0] - s + len(t[1])) for t in lines if search_text in t[3]]
        return matches
    
    def get_goto_action(self, r, c):
        index, _ = self.table.get_index_range(r, c)
        index_addr = self.table.get_pc(r)
        d = self.editor.disassembler(self.table.segment.data[index:], index_addr, -index_addr)
        next_addr, bytes, opstr, memloc, rw, addr_dest = d.disasm()
        segment_start = self.table.segment.start_addr
        segment_num = -1
        addr_index = addr_dest-segment_start
        if addr_dest is not None:
            if addr_dest < segment_start or addr_dest > segment_start + len(self.table.segment):
                segment_num, segment_dest, addr_index = self.editor.document.find_segment_in_range(addr_dest)
                if segment_dest is not None:
                    msg = "Go to address $%04x in segment %s" % (addr_dest, str(segment_dest))
                else:
                    msg = "Address $%04x not in any segment" % addr_dest
                    addr_dest = None
            else:
                msg = "Go to address $%04x" % addr_dest
        else:
            msg = "No address to jump to"
        if addr_dest is not None:
            goto_action = GotoIndexAction(name=msg, enabled=True, segment_num=segment_num, addr_index=addr_index, task=self.task, active_editor=self.task.active_editor)
        else:
            goto_action = GotoIndexAction(name=msg, enabled=False, task=self.task)
        return goto_action
    
    def get_popup_actions(self, r, c):
        goto_action = self.get_goto_action(r, c)
        return [goto_action, None, CutAction, CopyAction, PasteAction, None, SelectAllAction, SelectNoneAction, GetSegmentFromSelectionAction]



""" The default action controller for menus, menu bars and tool bars. """


# Enthought library imports.
from traits.api import HasTraits


class ActionController(HasTraits):
    """ The default action controller for menus, menu bars and tool bars. """

    ###########################################################################
    # 'ActionController' interface.
    ###########################################################################

    def perform(self, action, event):
        """ Control an action invocation. """

        return action.perform(event)

    def can_add_to_menu(self, action):
        """ Returns True if the action can be added to a menu/menubar. """

        return True

    def add_to_menu(self, action):
        """ Called when an action is added to the a menu/menubar. """

        return

    def can_add_to_toolbar(self, action):
        """ Returns True if the action can be added to a toolbar. """

        return True

    def add_to_toolbar(self, action):
        """ Called when an action is added to the a toolbar. """

        return

#### EOF ######################################################################



import sys
import wx

class SegmentList(wx.ListBox):
    """Segment selector for choosing which portion of the binary data to view
    """

    def __init__(self, parent, task, **kwargs):
        self.task = task
        
        wx.ListBox.__init__(self, parent, style=wx.LB_SINGLE, **kwargs)
        self.Bind(wx.EVT_LISTBOX, self.on_click)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.on_dclick)
        self.Bind(wx.EVT_RIGHT_DOWN, self.on_popup)
    
    def set_task(self, task):
        self.task = task
    
    def set_segments(self, segments, selected=0):
        items = [str(s) for s in segments]
        self.SetItems(items)
        self.SetSelection(selected)

    def on_click(self, event):
        selected = event.GetExtraLong()
        if selected:
            item = event.GetSelection()
            editor = self.task.active_editor
            print "Selected segment %d for document %s, control %s" % (item, editor.document, event.GetEventObject())
            wx.CallAfter(editor.view_segment_number, item)
        event.Skip()
    
    def on_dclick(self, event):
        event.Skip()
    
    def on_popup(self, event):
        event.Skip()



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" A window that is split in two either horizontally or vertically. """


# Local imports.
from application_window import ApplicationWindow
from split_widget import SplitWidget


class SplitApplicationWindow(ApplicationWindow, SplitWidget):
    """ A window that is split in two either horizontally or vertically. """

    ###########################################################################
    # Protected 'ApplicationWindow' interface.
    ###########################################################################

    def _create_contents(self, parent):
        """ Creates the window contents. """

        return self._create_splitter(parent)

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" A widget for editing Python code. """


# Enthought library imports.
from traits.api import Bool, Event, Interface, Unicode

# Local imports.
from key_pressed_event import KeyPressedEvent


class IPythonEditor(Interface):
    """ A widget for editing Python code. """

    #### 'IPythonEditor' interface ############################################

    # Has the file in the editor been modified?
    dirty = Bool(False)

    # The pathname of the file being edited.
    path = Unicode

    # Should line numbers be shown in the margin?
    show_line_numbers = Bool(True)

    #### Events ####

    # The contents of the editor has changed.
    changed = Event

    # A key has been pressed.
    key_pressed = Event(KeyPressedEvent)

    ###########################################################################
    # 'IPythonEditor' interface.
    ###########################################################################

    def load(self, path=None):
        """ Loads the contents of the editor. """

    def save(self, path=None):
        """ Saves the contents of the editor. """

    # FIXME v3: This is very dependent on the underlying implementation.
    def set_style(self, n, fore, back):
        """ Set the foreground and background colors for a particular style and
        set the font and size to default values.
        """

    def select_line(self, lineno):
        """ Selects the specified line. """


class MPythonEditor(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IPythonEditor interface.

    Implements: _changed_path()
    """

    def _changed_path(self):
        """ Called when the path to the file is changed. """

        if self.control is not None:
            self.load()

        return

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" The interface for a dialog that allows the user to open/save files etc. """

# Enthought library imports.
from traits.api import Any, Bool, Int, Str

# Local imports.
from i_dialog import IDialog


class IProgressDialog(IDialog):
    """ A simple progress dialog window which allows itself to be updated
    """

    #### 'IProgressDialog' interface ##################################
    title = Str
    message = Str
    min = Int
    max = Int
    margin = Int(5)
    can_cancel = Bool(False)
    show_time = Bool(False)
    show_percent = Bool(False)


    # Label for the 'cancel' button
    cancel_button_label = Str

    ###########################################################################
    # 'IProgressDialog' interface.
    ###########################################################################

    def update(self, value):
        """
        updates the progress bar to the desired value. If the value is >=
        the maximum and the progress bar is not contained in another panel
        the parent window will be closed

        """


class MProgressDialog(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IProgressDialog interface.

    Implements: update()
    """

    progress_bar = Any

    ###########################################################################
    # 'IProgressDialog' interface.
    ###########################################################################

    def update(self, value):
        """
        updates the progress bar to the desired value. If the value is >=
        the maximum and the progress bar is not contained in another panel
        the parent window will be closed

        """

        if self.progress_bar is not None:
            self.progress_bar.update(value)

        if value >= self.max:
            self.close()



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described
# in the PyQt GPL exception also apply

#
# Author: Riverbank Computing Limited
#------------------------------------------------------------------------------

""" Defines the various list editors for the PyQt user interface toolkit.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from pyface.qt import QtCore, QtGui

from pyface.api import ImageResource

from traits.api import Str, Any, Bool, Dict, Instance
from traits.trait_base import user_name_for, enumerate, xgetattr

# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.list_editor file.
from traitsui.editors.list_editor import ListItemProxy, \
    ToolkitEditorFactory

from editor import Editor
from helper import IconButton
from menu import MakeMenu

#-------------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------------

class SimpleEditor ( Editor ):
    """ Simple style of editor for lists, which displays a scrolling list box
    with only one item visible at a time. A icon next to the list box displays
    a menu of operations on the list.
    """

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # The kind of editor to create for each list item
    kind = Str

    # Is the list of items being edited mutable?
    mutable = Bool( True )

    # Signal mapper allowing to identify which icon button requested a context
    # menu
    mapper = Instance(QtCore.QSignalMapper)

    #---------------------------------------------------------------------------
    #  Class constants:
    #---------------------------------------------------------------------------

    # Whether the list is displayed in a single row
    single_row = True

    #---------------------------------------------------------------------------
    #  Normal list item menu:
    #---------------------------------------------------------------------------

    # Menu for modifying the list
    list_menu = """
       Add &Before     [_menu_before]: self.add_before()
       Add &After      [_menu_after]:  self.add_after()
       ---
       &Delete         [_menu_delete]: self.delete_item()
       ---
       Move &Up        [_menu_up]:     self.move_up()
       Move &Down      [_menu_down]:   self.move_down()
       Move to &Top    [_menu_top]:    self.move_top()
       Move to &Bottom [_menu_bottom]: self.move_bottom()
    """

    #---------------------------------------------------------------------------
    #  Empty list item menu:
    #---------------------------------------------------------------------------

    empty_list_menu = """
       Add: self.add_empty()
    """

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        # Initialize the trait handler to use:
        trait_handler = self.factory.trait_handler
        if trait_handler is None:
            trait_handler = self.object.base_trait( self.name ).handler
        self._trait_handler = trait_handler

        # Create a scrolled window to hold all of the list item controls:
        self.control = QtGui.QScrollArea()
        self.control.setFrameShape(QtGui.QFrame.NoFrame)
        self.control.setWidgetResizable(True)

        #Create a mapper to identify which icon button requested a contextmenu
        self.mapper = QtCore.QSignalMapper(self.control)

        # Create a widget with a grid layout as the container.
        self._list_pane = QtGui.QWidget()
        self._list_pane.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                      QtGui.QSizePolicy.Expanding)
        layout = QtGui.QGridLayout(self._list_pane)
        layout.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
        layout.setContentsMargins(0, 0, 0, 0)

        # Remember the editor to use for each individual list item:
        editor = self.factory.editor
        if editor is None:
            editor = trait_handler.item_trait.get_editor()
        self._editor = getattr( editor, self.kind )

        # Set up the additional 'list items changed' event handler needed for
        # a list based trait. Note that we want to fire the update_editor_item
        # only when the items in the list change and not when intermediate
        # traits change. Therefore, replace "." by ":" in the extended_name
        # when setting up the listener.
        extended_name = self.extended_name.replace('.', ':')
        self.context_object.on_trait_change( self.update_editor_item,
                               extended_name + '_items?', dispatch = 'ui' )
        self.set_tooltip()

    #---------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #---------------------------------------------------------------------------

    def dispose ( self ):
        """ Disposes of the contents of an editor.
        """
        self._dispose_items()

        extended_name = self.extended_name.replace('.', ':')
        self.context_object.on_trait_change( self.update_editor_item,
                                 extended_name + '_items?', remove = True )

        super( SimpleEditor, self ).dispose()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        self.mapper = QtCore.QSignalMapper(self.control)
        # Disconnect the editor from any control about to be destroyed:
        self._dispose_items()

        list_pane = self._list_pane
        layout = list_pane.layout()

        # Create all of the list item trait editors:
        trait_handler = self._trait_handler
        resizable     = ((trait_handler.minlen != trait_handler.maxlen) and
                         self.mutable)
        item_trait    = trait_handler.item_trait

        is_fake = (resizable and (len( self.value ) == 0))
        if is_fake:
            self.empty_list()
        else:
            # Asking the mapper to send the sender to the callback method
            self.mapper.mapped[QtCore.QObject].connect(self.popup_menu)

        editor = self._editor
        for index, value in enumerate(self.value):
            row, column = divmod(index, self.factory.columns)

            # Account for the fact that we have <columns> number of
            # pairs
            column = column * 2

            if resizable:
                # Connecting the new button to the mapper
                control = IconButton('list_editor.png', self.mapper.map)
                # Setting the mapping and asking it to send the index of the
                # sender to the callback method
                self.mapper.setMapping(control, control)

                layout.addWidget(control, row, column)

            proxy = ListItemProxy( self.object, self.name, index, item_trait,
                                   value )
            if resizable:
                control.proxy = proxy
            peditor = editor( self.ui, proxy, 'value', self.description,
                              list_pane ).set( object_name = '' )
            peditor.prepare( list_pane )
            pcontrol = peditor.control
            pcontrol.proxy = proxy

            if isinstance(pcontrol, QtGui.QWidget):
                layout.addWidget(pcontrol, row, column+1)
            else:
                layout.addLayout(pcontrol, row, column+1)

        # QScrollArea can have problems if the widget being scrolled is set too
        # early (ie. before it contains something).
        if self.control.widget() is None:
            self.control.setWidget(list_pane)

    #---------------------------------------------------------------------------
    #  Updates the editor when an item in the object trait changes external to
    #  the editor:
    #---------------------------------------------------------------------------

    def update_editor_item ( self, event ):
        """ Updates the editor when an item in the object trait changes
        externally to the editor.
        """
        # If this is not a simple, single item update, rebuild entire editor:
        if (len( event.removed ) != 1) or (len( event.added ) != 1):
            self.update_editor()
            return

        # Otherwise, find the proxy for this index and update it with the
        # changed value:
        for control in self.control.widget().children():
            if isinstance(control, QtGui.QLayout):
                continue

            proxy = control.proxy
            if proxy.index == event.index:
                proxy.value = event.added[0]
                break

    #---------------------------------------------------------------------------
    #  Creates an empty list entry (so the user can add a new item):
    #---------------------------------------------------------------------------

    def empty_list ( self ):
        """ Creates an empty list entry (so the user can add a new item).
        """
        # Connecting the new button to the mapper
        control = IconButton('list_editor.png', self.mapper.map)
        # Setting the mapping and asking it to send the sender to the
        # callback method
        self.mapper.setMapping(control, control)
        self.mapper.mapped[QtCore.QObject].connect(self.popup_empty_menu)
        control.is_empty = True
        self._cur_control = control

        proxy    = ListItemProxy( self.object, self.name, -1, None, None )
        pcontrol = QtGui.QLabel('   (Empty List)')
        pcontrol.proxy = control.proxy = proxy

        layout = self._list_pane.layout()
        layout.addWidget(control, 0, 0)
        layout.addWidget(pcontrol, 0, 1)

    #---------------------------------------------------------------------------
    #  Returns the associated object list and current item index:
    #---------------------------------------------------------------------------

    def get_info ( self ):
        """ Returns the associated object list and current item index.
        """
        proxy = self._cur_control.proxy
        return ( proxy.list, proxy.index )

    #---------------------------------------------------------------------------
    #  Displays the empty list editor popup menu:
    #---------------------------------------------------------------------------

    def popup_empty_menu ( self , sender):
        """ Displays the empty list editor popup menu.
        """
        self._cur_control = control = sender
        menu = MakeMenu( self.empty_list_menu, self, True, control ).menu
        menu.exec_(control.mapToGlobal(QtCore.QPoint(0, 0)))

    #---------------------------------------------------------------------------
    #  Displays the list editor popup menu:
    #---------------------------------------------------------------------------

    def popup_menu ( self , sender):
        """ Displays the list editor popup menu.
        """
        self._cur_control = sender

        proxy    = sender.proxy
        index    = proxy.index
        menu     = MakeMenu( self.list_menu, self, True, sender ).menu
        len_list = len( proxy.list )
        not_full = (len_list < self._trait_handler.maxlen)

        self._menu_before.enabled( not_full )
        self._menu_after.enabled(  not_full )
        self._menu_delete.enabled( len_list > self._trait_handler.minlen )
        self._menu_up.enabled(  index > 0 )
        self._menu_top.enabled( index > 0 )
        self._menu_down.enabled(   index < (len_list - 1) )
        self._menu_bottom.enabled( index < (len_list - 1) )

        menu.exec_(sender.mapToGlobal(QtCore.QPoint(0, 0)))

    #---------------------------------------------------------------------------
    #  Adds a new value at the specified list index:
    #---------------------------------------------------------------------------

    def add_item ( self, offset ):
        """ Adds a new value at the specified list index.
        """
        list, index = self.get_info()
        index      += offset
        item_trait  = self._trait_handler.item_trait
        value       = item_trait.default_value_for( self.object, self.name )
        self.value = list[:index] + [ value ] + list[index:]
        self.update_editor()

    #---------------------------------------------------------------------------
    #  Inserts a new item before the current item:
    #---------------------------------------------------------------------------

    def add_before ( self ):
        """ Inserts a new item before the current item.
        """
        self.add_item( 0 )

    #---------------------------------------------------------------------------
    #  Inserts a new item after the current item:
    #---------------------------------------------------------------------------

    def add_after ( self ):
        """ Inserts a new item after the current item.
        """
        self.add_item( 1 )

    #---------------------------------------------------------------------------
    #  Adds a new item when the list is empty:
    #---------------------------------------------------------------------------

    def add_empty ( self ):
        """ Adds a new item when the list is empty.
        """
        list, index = self.get_info()
        self.add_item( 0 )

    #---------------------------------------------------------------------------
    #  Delete the current item:
    #---------------------------------------------------------------------------

    def delete_item ( self ):
        """ Delete the current item.
        """
        list, index = self.get_info()
        self.value  = list[:index] + list[index+1:]
        self.update_editor()

    #---------------------------------------------------------------------------
    #  Move the current item up one in the list:
    #---------------------------------------------------------------------------

    def move_up ( self ):
        """ Move the current item up one in the list.
        """
        list, index = self.get_info()
        self.value  = (list[:index-1] + [ list[index], list[index-1] ] +
                       list[index+1:])
        self.update_editor()

    #---------------------------------------------------------------------------
    #  Moves the current item down one in the list:
    #---------------------------------------------------------------------------

    def move_down ( self ):
        """ Moves the current item down one in the list.
        """
        list, index = self.get_info()
        self.value  = (list[:index] + [ list[index+1], list[index] ] +
                       list[index+2:])
        self.update_editor()

    #---------------------------------------------------------------------------
    #  Moves the current item to the top of the list:
    #---------------------------------------------------------------------------

    def move_top ( self ):
        """ Moves the current item to the top of the list.
        """
        list, index = self.get_info()
        self.value  = [ list[index] ] + list[:index] + list[index+1:]
        self.update_editor()

    #---------------------------------------------------------------------------
    #  Moves the current item to the bottom of the list:
    #---------------------------------------------------------------------------

    def move_bottom ( self ):
        """ Moves the current item to the bottom of the list.
        """
        list, index = self.get_info()
        self.value  = list[:index] + list[index+1:] + [ list[index] ]
        self.update_editor()

    #-- Private Methods --------------------------------------------------------

    def _dispose_items ( self ):
        """ Disposes of each current list item.
        """
        layout = self._list_pane.layout()
        child = layout.takeAt(0)
        while child is not None:
            control = child.widget()
            if control is not None:
                editor = getattr( control, '_editor', None )
                if editor is not None:
                    editor.dispose()
                    editor.control = None
                control.deleteLater()
            child = layout.takeAt(0)
        del child

    #-- Trait initializers ----------------------------------------------------

    def _kind_default(self):
        """ Returns a default value for the 'kind' trait.
        """
        return self.factory.style + '_editor'

    def _mutable_default(self):
        """ Trait handler to set the mutable trait from the factory.
        """
        return self.factory.mutable

#-------------------------------------------------------------------------------
#  'CustomEditor' class:
#-------------------------------------------------------------------------------

class CustomEditor ( SimpleEditor ):
    """ Custom style of editor for lists, which displays the items as a series
    of text fields. If the list is editable, an icon next to each item displays
    a menu of operations on the list.
    """

    #---------------------------------------------------------------------------
    #  Class constants:
    #---------------------------------------------------------------------------

    # Whether the list is displayed in a single row. This value overrides the
    # default.
    single_row = False

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Is the list editor is scrollable? This values overrides the default.
    scrollable = True

#-------------------------------------------------------------------------------
#  'TextEditor' class:
#-------------------------------------------------------------------------------

class TextEditor(CustomEditor):

    # The kind of editor to create for each list item. This value overrides the
    # default.
    kind = 'text_editor'

#-------------------------------------------------------------------------------
#  'ReadonlyEditor' class:
#-------------------------------------------------------------------------------

class ReadonlyEditor(CustomEditor):

    # Is the list of items being edited mutable? This value overrides the
    # default.
    mutable = False

#-------------------------------------------------------------------------------
#  'NotebookEditor' class:
#-------------------------------------------------------------------------------

class NotebookEditor ( Editor ):
    """ An editor for lists that displays the list as a "notebook" of tabbed
    pages.
    """

    # The "Close Tab" button.
    close_button = Any()

    # Maps tab names to QWidgets representing the tab contents
    # TODO: It would be nice to be able to reuse self._pages for this, but
    # its keys are not quite what we want.
    _pagewidgets = Dict

    # Maps names of tabs to their menu QAction instances; used to toggle
    # checkboxes
    _action_dict = Dict


    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Is the notebook editor scrollable? This values overrides the default:
    scrollable = True

    # The currently selected notebook page object:
    selected = Any

    # Maps tab names to QWidgets representing the tab contents
    # TODO: It would be nice to be able to reuse self._pages for this, but
    # its keys are not quite what we want.
    _pagewidgets = Dict

    # Maps names of tabs to their menu QAction instances; used to toggle
    # checkboxes
    _action_dict = Dict

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self._uis = []

        # Create a tab widget to hold each separate object's view:
        self.control = QtGui.QTabWidget()
        signal = QtCore.SIGNAL( 'currentChanged(int)' )
        QtCore.QObject.connect( self.control, signal, self._tab_activated )

        # Create the button to close tabs, if necessary:
        if self.factory.deletable:
            button = QtGui.QToolButton()
            button.setAutoRaise( True )
            button.setToolTip( 'Remove current tab ')
            button.setIcon ( ImageResource( 'closetab' ).create_icon() )

            self.control.setCornerWidget( button, QtCore.Qt.TopRightCorner )
            signal = QtCore.SIGNAL( 'clicked()' )
            QtCore.QObject.connect( button, signal, self.close_current )
            self.close_button = button

        if self.factory.show_notebook_menu:
            # Create the necessary attributes to manage hiding and revealing of
            # tabs via a context menu
            self._context_menu = QtGui.QMenu()
            self.control.customContextMenuRequested.connect(self._context_menu_requested)
            self.control.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)

        # Set up the additional 'list items changed' event handler needed for
        # a list based trait:
        self.context_object.on_trait_change( self.update_editor_item,
                               self.extended_name + '_items?', dispatch = 'ui' )

        # Set of selection synchronization:
        self.sync_value( self.factory.selected, 'selected' )

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        # Destroy the views on each current notebook page:
        self.close_all()

        # Create a tab page for each object in the trait's value:
        for object in self.value:
            ui, view_object, monitoring = self._create_page(object)

            # Remember the page for later deletion processing:
            self._uis.append([ui.control, ui, view_object, monitoring])

        if self.selected:
            self._selected_changed(self.selected)

    #---------------------------------------------------------------------------
    #  Handles some subset of the trait's list being updated:
    #---------------------------------------------------------------------------

    def update_editor_item ( self, event ):
        """ Handles an update to some subset of the trait's list.
        """
        index = event.index

        # Delete the page corresponding to each removed item:
        page_name = self.factory.page_name[1:]

        for i in event.removed:
            page, ui, view_object, monitoring = self._uis[index]
            if monitoring:
                view_object.on_trait_change(self.update_page_name, page_name,
                        remove=True)
            ui.dispose()
            self.control.removeTab(self.control.indexOf(page))

            if self.factory.show_notebook_menu:
                for name, tmp in self._pagewidgets.items():
                    if tmp is page:
                        del self._pagewidgets[name]
                self._context_menu.removeAction(self._action_dict[name])
                del self._action_dict[name]

            del self._uis[index]

        # Add a page for each added object:
        first_page = None
        for object in event.added:
            ui, view_object, monitoring  = self._create_page(object)
            self._uis[index:index] = [[ui.control, ui, view_object, monitoring]]
            index += 1

            if first_page is None:
                first_page = ui.control

        if first_page is not None:
            self.control.setCurrentWidget(first_page)

    #---------------------------------------------------------------------------
    #  Closes the currently selected tab:
    #---------------------------------------------------------------------------

    def close_current ( self, force=False ):
        """ Closes the currently selected tab:
        """
        widget = self.control.currentWidget()
        for i in xrange( len( self._uis ) ):
            page, ui, _, _ = self._uis[i]
            if page is widget:
                if force or ui.handler.close( ui.info, True ):
                    del self.value[i]
                break

        if self.factory.show_notebook_menu:
            # Find the name associated with this widget, so we can purge its action
            # from the menu
            for name, tmp in self._pagewidgets.items():
                if tmp is widget:
                    break
            else:
                # Hmm... couldn't find the widget, assume that we don't need to do
                # anything.
                return

            action = self._action_dict[name]
            self._context_menu.removeAction(action)
            del self._action_dict[name]
            del self._pagewidgets[name]
        return

    #---------------------------------------------------------------------------
    #  Closes all currently open notebook pages:
    #---------------------------------------------------------------------------

    def close_all ( self ):
        """ Closes all currently open notebook pages.
        """
        page_name = self.factory.page_name[1:]

        for _, ui, view_object, monitoring in self._uis:
            if monitoring:
                view_object.on_trait_change(self.update_page_name, page_name,
                        remove=True)
            ui.dispose()

        # Reset the list of ui's and dictionary of page name counts:
        self._uis = []
        self._pages = {}

        self.control.clear()

    #---------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #---------------------------------------------------------------------------

    def dispose ( self ):
        """ Disposes of the contents of an editor.
        """
        self.context_object.on_trait_change( self.update_editor_item,
                                self.name + '_items?', remove = True )
        self.close_all()

        super( NotebookEditor, self ).dispose()

    #---------------------------------------------------------------------------
    #  Handles the trait defining a particular page's name being changed:
    #---------------------------------------------------------------------------

    def update_page_name ( self, object, name, old, new ):
        """ Handles the trait defining a particular page's name being changed.
        """
        for i, value in enumerate(self._uis):
            page, ui, _, _ = value
            if object is ui.info.object:
                name = None
                handler = getattr(self.ui.handler,
                        '%s_%s_page_name' % (self.object_name, self.name),
                        None)

                if handler is not None:
                    name = handler(self.ui.info, object)

                if name is None:
                    name = str(xgetattr(object, self.factory.page_name[1:], '???'))
                self.control.setTabText(self.control.indexOf(page), name)
                break

    #---------------------------------------------------------------------------
    #  Creates a page for a specified object and adds it to the tab widget:
    #---------------------------------------------------------------------------

    def _create_page ( self, object ):
        # Create the view for the object:
        view_object = object
        factory = self.factory
        if factory.factory is not None:
            view_object = factory.factory(object)
        ui = view_object.edit_traits( parent = self.control,
                                 view   = factory.view,
                                 kind   = factory.ui_kind ).set(
                                 parent = self.ui )

        # Get the name of the page being added to the notebook:
        name       = ''
        monitoring = False
        prefix     = '%s_%s_page_' % ( self.object_name, self.name )
        page_name  = factory.page_name
        if page_name[0:1] == '.':
            name       = xgetattr( view_object, page_name[1:], None )
            monitoring = (name is not None)
            if monitoring:
                handler_name = None
                method       = getattr( self.ui.handler, prefix + 'name', None )
                if method is not None:
                    handler_name = method( self.ui.info, object )
                if handler_name is not None:
                    name = handler_name
                else:
                    name = str( name ) or '???'
                view_object.on_trait_change( self.update_page_name,
                                        page_name[1:], dispatch = 'ui' )
            else:
                name = ''
        elif page_name != '':
            name = page_name

        if name == '':
            name = user_name_for( view_object.__class__.__name__ )

        # Make sure the name is not a duplicate:
        if not monitoring:
            self._pages[ name ] = count = self._pages.get( name, 0 ) + 1
            if count > 1:
                name += (' %d' % count)

        # Return the control for the ui, and whether or not its name is being
        # monitored:
        image   = None
        method  = getattr( self.ui.handler, prefix + 'image', None )
        if method is not None:
            image = method( self.ui.info, object )

        if image is None:
            self.control.addTab(ui.control, name)
        else:
            self.control.addTab(ui.control, image, name)

        if self.factory.show_notebook_menu:
            newaction = self._context_menu.addAction(name)
            newaction.setText(name)
            newaction.setCheckable(True)
            newaction.setChecked(True)
            newaction.triggered.connect(lambda e,name=name: self._menu_action(e,name=name))
            self._action_dict[name] = newaction
            self._pagewidgets[name] = ui.control

        return (ui, view_object, monitoring)

    def _tab_activated(self, idx):
        """ Handles a notebook tab being "activated" (i.e. clicked on) by the
            user.
        """
        widget = self.control.widget(idx)
        for page, ui, _, _ in self._uis:
            if page is widget:
                self.selected = ui.info.object
                break

    def _selected_changed(self, selected):
        """ Handles the **selected** trait being changed.
        """
        for page, ui, _, _ in self._uis:
            if ui.info and selected is ui.info.object:
                self.control.setCurrentWidget(page)
                break
            deletable = self.factory.deletable
            deletable_trait = self.factory.deletable_trait
            if deletable and deletable_trait:
                enabled = xgetattr(selected, deletable_trait, True)
                self.close_button.setEnabled(enabled)

    def _context_menu_requested(self, event):
        self._context_menu.popup(self.control.mapToGlobal(event))

    def _menu_action(self, event, name=""):
        """ Qt signal handler for when a item in a context menu is actually
        selected.  Not that we get this even after the underlying value has
        already changed.
        """
        action = self._action_dict[name]
        checked = action.isChecked()
        if not checked:
            for ndx in range(self.control.count()):
                if self.control.tabText(ndx) == name:
                    self.control.removeTab(ndx)
        else:
            # TODO: Fix tab order based on the context_object's list
            self.control.addTab(self._pagewidgets[name], name)

    def _context_menu_requested(self, event):
        self._context_menu.popup(self.control.mapToGlobal(event))

    def _menu_action(self, event, name=""):
        """ Qt signal handler for when a item in a context menu is actually
        selected.  Not that we get this even after the underlying value has
        already changed.
        """
        action = self._action_dict[name]
        checked = action.isChecked()
        if not checked:
            for ndx in range(self.control.count()):
                if self.control.tabText(ndx) == name:
                    self.control.removeTab(ndx)
        else:
            # TODO: Fix tab order based on the context_object's list
            self.control.addTab(self._pagewidgets[name], name)



#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from __future__ import absolute_import

from traits.api import Float, Enum, Any, Property

from ..view import View

from ..item import Item

from ..editor_factory import EditorFactory

from ..basic_editor_factory import BasicEditorFactory

from .text_editor import TextEditor

from ..ui_traits import EditorStyle

from ..ui_editor import UIEditor

from ..toolkit import toolkit_object

#-------------------------------------------------------------------------------
#  '_PopupEditor' class:
#-------------------------------------------------------------------------------

class _PopupEditor ( UIEditor ):

    #---------------------------------------------------------------------------
    #  Creates the traits UI for the editor:
    #---------------------------------------------------------------------------

    def init_ui ( self, parent ):
        """ Creates the traits UI for the editor.
        """
        return self.object.edit_traits( view   = self.base_view(),
                                        parent = parent )

    def base_view ( self ):
        """ Returns the View that allows the popup view to be displayed.
        """
        return View(
            Item( self.name,
                  show_label = False,
                  style      = 'readonly',
                  editor     = TextEditor( view = self.popup_view() ),
                  padding    = -4,
            ),
            kind = 'subpanel'
        )

    def popup_view ( self ):
        """ Returns the popup View.
        """
        factory = self.factory
        item    = Item( self.name,
                        show_label = False,
                        padding    = -4,
                        style      = factory.style,
                        height     = factory.height,
                        width      = factory.width )

        editor = factory.editor
        if editor is not None:
            if not isinstance( editor, EditorFactory ):
                editor = editor()
            item.editor = editor

        return View( item, kind = factory.kind )

#-------------------------------------------------------------------------------
#  'PopupEditor' class:
#-------------------------------------------------------------------------------

class PopupEditor ( BasicEditorFactory ):

    # The class used to construct editor objects:
    klass = Property

    # The kind of popup to use:
    kind = Enum( 'popover', 'popup', 'info' )

    # The editor to use for the pop-up view (can be None (use default editor),
    # an EditorFactory instance, or a callable that returns an EditorFactory
    # instance):
    editor = Any

    # The style of editor to use for the popup editor (same as Item.style):
    style = EditorStyle

    # The height of the popup (same as Item.height):
    height = Float( -1.0 )

    # The width of the popup (same as Item.width):
    width = Float( -1.0 )

    def _get_klass( self ):
        """ The class used to construct editor objects.
        """
        return toolkit_object('popup_editor:_PopupEditor')



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" The interface for an image cache. """


# Enthought library imports.
from traits.api import Interface


class IImageCache(Interface):
    """ The interface for an image cache. """

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, width, height):
        """ Creates a new image cache for images of the given size. """

    ###########################################################################
    # 'ImageCache' interface.
    ###########################################################################

    def get_image(self, filename):
        """ Returns the specified image. """

    # FIXME v3: The need to distinguish between bitmaps and images is toolkit
    # specific so, strictly speaking, the conversion to a bitmap should be done
    # wherever the toolkit actually needs it.
    def get_bitmap(self, filename):
        """ Returns the specified image as a bitmap. """


class MImageCache(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IImageCache interface.
    """

#### EOF ######################################################################



#! /usr/bin/env python3
#
# Disassembler for Intel 8080 microprocessor.
# Copyright (c) 2013-2015 by Jeff Tranter <tranter@pobox.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Possible enhancements:
# - Read Intel HEX file format
# - binary (front panel) output mode, e.g.
#
#       ADDRESS              DATA
# . ... ... ... ... ...   .. ... ...
# 0 001 001 000 110 100   11 000 011  JMP
# 0 001 001 000 110 101   00 110 100  064
# 0 001 001 000 110 110   00 001 001  011

import sys
import fileinput
import argparse
import signal

# Avoids an error when output piped, e.g. to "less"
signal.signal(signal.SIGPIPE, signal.SIG_DFL)

# Lookup table - given opcode byte as index, return mnemonic of instruction
# and length of instruction.

# Alternative opcodes have a '*' prepended.
lookupTable = [
    ["nop",         1],  # 00
    ["lxi     b,",  3],  # 01
    ["stax    b",   1],  # 02
    ["inx     b",   1],  # 03
    ["inr     b",   1],  # 04
    ["dcr     b",   1],  # 05
    ["mvi     b,",  2],  # 06
    ["rlc",         1],  # 07
    ["*nop",        1],  # 08
    ["dad     b",   1],  # 09
    ["ldax    b",   1],  # 0A
    ["dcx     b",   1],  # 0B
    ["inr     c",   1],  # 0C
    ["dcr     c",   1],  # 0D
    ["mvi     c,",  2],  # 0E
    ["rrc",         1],  # 0F

    ["*nop",        1],  # 10
    ["lxi     d,",  3],  # 11
    ["stax    d",   1],  # 12
    ["inx     d",   1],  # 13
    ["inr     d",   1],  # 14
    ["dcr     d",   1],  # 15
    ["mvi     d,",  2],  # 16
    ["ral",         1],  # 17
    ["*nop",        1],  # 18
    ["dad",         1],  # 19
    ["ldax    d",   1],  # 1A
    ["dcx     d",   1],  # 1B
    ["inr     e",   1],  # 1C
    ["dcr     e",   1],  # 1D
    ["mvi     e,",  2],  # 1E
    ["rar",         1],  # 1F

    ["*nop",        1],  # 20
    ["lxi     h,",  3],  # 21
    ["shld    ",    3],  # 22
    ["inx     h",   1],  # 23
    ["inr     h",   1],  # 24
    ["dcr     h",   1],  # 25
    ["mvi     h,",  2],  # 26
    ["daa",         1],  # 27
    ["*nop",        1],  # 28
    ["dad     h",   1],  # 29
    ["lhld    ",    3],  # 2A
    ["dcx     h",   1],  # 2B
    ["inr     l",   1],  # 2C
    ["dcr     l",   1],  # 2D
    ["mvi     l,",  2],  # 2E
    ["cma",         1],  # 2F

    ["*nop",        1],  # 30
    ["lxi     sp,", 3],  # 31
    ["sta     ",    3],  # 32
    ["inx     sp",  1],  # 33
    ["inr     m",   1],  # 34
    ["dcr     m",   1],  # 35
    ["mvi     m,",  2],  # 36
    ["stc",         1],  # 37
    ["*nop",        1],  # 38
    ["dad     sp",  1],  # 39
    ["lda     ",    3],  # 3A
    ["dcx     sp",  1],  # 3B
    ["inr     a",   1],  # 3C
    ["dcr     a",   1],  # 3D
    ["mvi     a,",  2],  # 3E
    ["cmc",         1],  # 3F

    ["mov     b,b", 1],  # 40
    ["mov     b,c", 1],  # 41
    ["mov     b,d", 1],  # 42
    ["mov     b,e", 1],  # 43
    ["mov     b,h", 1],  # 44
    ["mov     b,l", 1],  # 45
    ["mov     b,m", 1],  # 46
    ["mov     b,a", 1],  # 47
    ["mov     c,b", 1],  # 48
    ["mov     c,c", 1],  # 49
    ["mov     c,d", 1],  # 4A
    ["mov     c,e", 1],  # 4B
    ["mov     c,h", 1],  # 4C
    ["mov     c,l", 1],  # 4D
    ["mov     c,m", 1],  # 4E
    ["mov     c,a", 1],  # 4F

    ["mov     d,b", 1],  # 50
    ["mov     d,c", 1],  # 51
    ["mov     d,d", 1],  # 52
    ["mov     d,e", 1],  # 53
    ["mov     d,h", 1],  # 54
    ["mov     d,l", 1],  # 55
    ["mov     d,m", 1],  # 56
    ["mov     d,a", 1],  # 57
    ["mov     e,b", 1],  # 58
    ["mov     e,c", 1],  # 59
    ["mov     e,d", 1],  # 5A
    ["mov     e,e", 1],  # 5B
    ["mov     e,h", 1],  # 5C
    ["mov     e,l", 1],  # 5D
    ["mov     e,m", 1],  # 5E
    ["mov     e,a", 1],  # 5F

    ["mov     h,b", 1],  # 60
    ["mov     h,c", 1],  # 61
    ["mov     h,d", 1],  # 62
    ["mov     h,e", 1],  # 63
    ["mov     h,h", 1],  # 64
    ["mov     h,l", 1],  # 65
    ["mov     h,m", 1],  # 66
    ["mov     h,a", 1],  # 67
    ["mov     l,b", 1],  # 68
    ["mov     l,c", 1],  # 69
    ["mov     l,d", 1],  # 6A
    ["mov     l,e", 1],  # 6B
    ["mov     l,h", 1],  # 6C
    ["mov     l,l", 1],  # 6D
    ["mov     l,m", 1],  # 6E
    ["mov     l,a", 1],  # 6F

    ["mov     m,b", 1],  # 70
    ["mov     m,c", 1],  # 71
    ["mov     m,d", 1],  # 72
    ["mov     m,e", 1],  # 73
    ["mov     m,h", 1],  # 74
    ["mov     m,l", 1],  # 75
    ["hlt",         1],  # 76
    ["mov     m,a", 1],  # 77
    ["mov     a,b", 1],  # 78
    ["mov     a,c", 1],  # 79
    ["mov     a,d", 1],  # 7A
    ["mov     a,e", 1],  # 7B
    ["mov     a,h", 1],  # 7C
    ["mov     a,l", 1],  # 7D
    ["mov     a,m", 1],  # 7E
    ["mov     a,a", 1],  # 7F

    ["add     b",   1],  # 80
    ["add     c",   1],  # 81
    ["add     d",   1],  # 82
    ["add     e",   1],  # 83
    ["add     h",   1],  # 84
    ["add     l",   1],  # 85
    ["add     m",   1],  # 86
    ["add     a",   1],  # 87
    ["adc     b",   1],  # 88
    ["adc     c",   1],  # 89
    ["adc     d",   1],  # 8A
    ["adc     e",   1],  # 8B
    ["adc     h",   1],  # 8C
    ["adc     l",   1],  # 8D
    ["adc     m",   1],  # 8E
    ["adc     a",   1],  # 8F

    ["sub     b",   1],  # 90
    ["sub     c",   1],  # 91
    ["sub     d",   1],  # 92
    ["sub     e",   1],  # 93
    ["sub     h",   1],  # 94
    ["sub     l",   1],  # 95
    ["sub     m",   1],  # 96
    ["sub     a",   1],  # 97
    ["sbb     b",   1],  # 98
    ["sbb     c",   1],  # 99
    ["sbb     d",   1],  # 9A
    ["sbb     e",   1],  # 9B
    ["sbb     h",   1],  # 9C
    ["sbb     l",   1],  # 9D
    ["sbb     m",   1],  # 9E
    ["sbb     a",   1],  # 9F

    ["ana     b",   1],  # A0
    ["ana     c",   1],  # A1
    ["ana     d",   1],  # A2
    ["ana     e",   1],  # A3
    ["ana     h",   1],  # A4
    ["ana     l",   1],  # A5
    ["ana     m",   1],  # A6
    ["ana     a",   1],  # A7
    ["xra     b",   1],  # A8
    ["xra     c",   1],  # A9
    ["xra     d",   1],  # AA
    ["xra     e",   1],  # AB
    ["xra     h",   1],  # AC
    ["xra     l",   1],  # AD
    ["xra     m",   1],  # AE
    ["xra     a",   1],  # AF

    ["ora     b",   1],  # B0
    ["ora     c",   1],  # B1
    ["ora     d",   1],  # B2
    ["ora     e",   1],  # B3
    ["ora     h",   1],  # B4
    ["ora     l",   1],  # B5
    ["ora     m",   1],  # B6
    ["ora     a",   1],  # B7
    ["cmp     b",   1],  # B8
    ["cmp     c",   1],  # B9
    ["cmp     d",   1],  # BA
    ["cmp     e",   1],  # BB
    ["cmp     h",   1],  # BC
    ["cmp     l",   1],  # BD
    ["cmp     m",   1],  # BE
    ["cmp     a",   1],  # BF

    ["rnz",         1],  # C0
    ["pop     b",   1],  # C1
    ["jnz     ",    3],  # C2
    ["jmp     ",    3],  # C3
    ["cnz     ",    3],  # C4
    ["push    b",   1],  # C5
    ["adi     ",    2],  # C6
    ["rst     0",   1],  # C7
    ["rz",          1],  # C8
    ["ret",         1],  # C9
    ["jz      ",    3],  # CA
    ["*jmp     ",   3],  # CB
    ["cz      ",    3],  # CC
    ["call    ",    3],  # CD
    ["aci     ",    2],  # CE
    ["rst     1",   1],  # CF

    ["rnc",         1],  # D0
    ["pop     d",   1],  # D1
    ["jnc     ",    3],  # D2
    ["out     ",    2],  # D3
    ["cnc     ",    3],  # D4
    ["push    d",   1],  # D5
    ["sui     ",    2],  # D6
    ["rst     2",   1],  # D7
    ["rc",          1],  # D8
    ["*ret",        1],  # D9
    ["jc      ",    3],  # DA
    ["in      ",    2],  # DB
    ["cc      ",    3],  # DC
    ["*call    ",   3],  # DD
    ["sbi     ",    2],  # DE
    ["rst     3",   1],  # DF

    ["rpo",         1],  # E0
    ["pop     h",   1],  # E1
    ["jpo     ",    3],  # E2
    ["xthl",        1],  # E3
    ["cpo     ",    3],  # E4
    ["push    h",   1],  # E5
    ["ani     ",    2],  # E6
    ["rst     4",   1],  # E7
    ["rpe",         1],  # E8
    ["pchl",        1],  # E9
    ["jpe     ",    3],  # EA
    ["xchg",        1],  # EB
    ["cpe     ",    3],  # EC
    ["*call    ",   3],  # ED
    ["xri     ",    2],  # EE
    ["rst     5",   1],  # EF

    ["rp",          1],  # F0
    ["pop     psw", 1],  # F1
    ["jp      ",    3],  # F2
    ["di",          1],  # F3
    ["cp      ",    3],  # F4
    ["push    psw", 1],  # F5
    ["ori     ",    2],  # F6
    ["rst     6",   1],  # F7
    ["rm",          1],  # F8
    ["sphl",        1],  # F9
    ["jm      ",    3],  # FA
    ["ei",          1],  # FB
    ["cm      ",    3],  # FC
    ["*call    ",   3],  # FD
    ["cpi     ",    2],  # FE
    ["rst     7",   1],  # FF
]

# Indicates if uppercase option is in effect.
upperOption = False

# Functions


def isprint(c):
    "Return if character is printable ASCII"
    if c >= '@' and c <= '~':
        return True
    else:
        return False


def case(s):
    "Return string or uppercase version of string if option is set."
    global upperOption
    if upperOption:
        return s.upper()
    else:
        return s


def formatByte(data):
    "Format an 8-bit byte using the current display format (e.g. hex or octal)"
    global args
    if args.format == 4:  # Octal
        return "%03o" % data
    else:  # Hex
        return "%02X" % data


def formatAddress(data):
    "Format a 16-bit address using the current display format (e.g. hex or octal)"
    global args
    if args.format == 4:  # Octal
        return "%06o" % data
    else:  # Hex
        return "%04X" % data

# Parse command line options
parser = argparse.ArgumentParser()
parser.add_argument("filename", help="Binary file to disassemble")
parser.add_argument("-n", "--nolist",
                    help="Don't list  instruction bytes (make output suitable for assembler)",
                    action="store_true")
parser.add_argument("-u", "--uppercase", help="Use uppercase for mnemonics", action="store_true")
parser.add_argument("-a", "--address", help="Specify decimal starting address (defaults to 0)", default=0, type=int)
parser.add_argument("-f", "--format",
                    help="Use number format: 1=$1234 2=1234h 3=1234 4=177777 (default 1)",
                    default=1, type=int, choices=range(1, 5))
args = parser.parse_args()

# Get filename from command line arguments.
filename = args.filename

# Current instruction address. Silently force it to be in valid range.
address = args.address & 0xffff

# Set uppercase output option.
upperOption = args.uppercase

# Contains a line of output
line = ""

# Open input file.
# Display error and exit if filename does not exist.
try:
    f = open(filename, "rb")
except FileNotFoundError:
    print("error: input file '%s' not found." % filename, file=sys.stderr)
    sys.exit(1)

# Print initial origin address
if args.nolist is False:
    if args.format == 1:
        print("%04X            %s     $%04X" % (address, case("org"), address))
    elif args.format == 2:
        print("%04X            %s     %04X%s" % (address, case("org"), address, case("h")))
    elif args.format == 3:
        print("%04X            %s     %04X" % (address, case("org"), address))
    else:
        print("%06o               %s     %06o" % (address, case("org"), address))

while True:
    try:
        b = f.read(1)  # Get binary byte from file

        if len(b) == 0:  # EOF
            if args.nolist is False:
                if args.format == 4:
                    print("%06o               %s" % (address, case("end")))  # Exit if end of file reached.
                else:
                    print("%04X            %s" % (address, case("end")))  # Exit if end of file reached.
            break

        if args.nolist is False:
            line = "%s  " % formatAddress(address)  # Print current address

        op = ord(b)  # Get opcode byte

        n = lookupTable[op][1]  # Look up number of instruction bytes

        mnem = case(lookupTable[op][0])  # Get mnemonic

        # Print instruction bytes
        if n == 1:
            if args.nolist is False:
                if args.format == 4:
                    line += "%03o          " % op
                else:
                    line += "%02X        " % op
        elif n == 2:
            try:  # Possible to get exception here if EOF reached.
                op1 = ord(f.read(1))
            except TypeError:
                op1 = 0  # Fake it to recover from EOF
            if args.nolist is False:
                if args.format == 4:
                    line += "%03o %03o      " % (op, op1)
                else:
                    line += "%02X %02X     " % (op, op1)
        elif n == 3:
            try:  # Possible to get exception here if EOF reached.
                op1 = ord(f.read(1))
                op2 = ord(f.read(1))
            except TypeError:
                op1 = 0  # Fake it to recover from EOF
                op2 = 0
            if args.nolist is False:
                line += "%s %s %s  " % (formatByte(op), formatByte(op1), formatByte(op2))
        if args.nolist is True:
            line += " "

        # If opcode starts with '*' then put in comment that this is an alternative op code (likely an error).
        if mnem[0] == "*":
            alternative = True
            mnem = mnem.replace(mnem[:1], '')  # Remove the star
        else:
            alternative = False

        line += mnem

        # Handle any operands
        if n == 2:
            if isprint(chr(op1)):
                line += "'%c'" % op1
            else:
                if args.format == 1:
                    line += "$%s" % formatByte(op1)
                elif args.format == 2:
                    line += "%s%s" % (formatByte(op1), case("h"))
                else:
                    line += "%s" % formatByte(op1)
        elif n == 3:
            if args.format == 1:
                line += "$%s%s" % (formatByte(op2), formatByte(op1))
            elif args.format == 2:
                line += "%s%s%s" % (formatByte(op2), formatByte(op1), case("h"))
            else:
                line += "%s%s" % (formatByte(op2), formatByte(op1))

        if alternative:
            mnem = mnem.replace(mnem[:1], '')  # Remove the star
            # Line up comment at fixed column position
            if args.nolist is False:
                line += ";Note: Alternative opcode used".rjust(67 - len(line))
            else:
                line += ";Note: Alternative opcode used".rjust(51 - len(line))

        # Update address
        address += n

        # Check for address exceeding 0xFFFF, if so wrap around.
        if address > 0xffff:
            address = address & 0xffff

        # Finished a line of disassembly
        print(line)
        line = ""

    except KeyboardInterrupt:
        # Exit if end of file reached.
        print("Interrupted by Control-C", file=sys.stderr)
        if args.format == 4:
            print("%s               %s" % (formatAddress(address), case("end")))
        else:
            print("%s            %s" % (formatAddress(address), case("end")))
        break



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/29/2004
#
#------------------------------------------------------------------------------

""" Defines a wxPython ImageControl widget that is used by various trait
    editors to display trait values iconically.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

#-------------------------------------------------------------------------------
#  'ImageControl' class:
#-------------------------------------------------------------------------------

class ImageControl ( wx.Window ):
    """ A wxPython control that displays an image, which can be selected or
        unselected by mouse clicks.
    """

    # Pens used to draw the 'selection' marker:
    _selectedPenDark = wx.Pen(
        wx.SystemSettings_GetColour( wx.SYS_COLOUR_3DSHADOW ), 1,
        wx.SOLID )

    _selectedPenLight = wx.Pen(
        wx.SystemSettings_GetColour( wx.SYS_COLOUR_3DHIGHLIGHT ), 1,
        wx.SOLID )

    #---------------------------------------------------------------------------
    #  Initializes the object:
    #---------------------------------------------------------------------------

    def __init__ ( self, parent, bitmap, selected = None, handler = None,
                         padding = 10 ):
        """ Initializes the object.
        """
        wx.Window.__init__( self, parent, -1,
                            size = wx.Size( bitmap.GetWidth()  + padding,
                                            bitmap.GetHeight() + padding ) )
        self._bitmap      = bitmap
        self._selected    = selected
        self._handler     = handler
        self._mouse_over  = False
        self._button_down = False

        # Set up the 'paint' event handler:
        wx.EVT_PAINT( self, self._on_paint )

        # Set up mouse event handlers:
        wx.EVT_LEFT_DOWN(    self, self._on_left_down )
        wx.EVT_LEFT_UP(      self, self._on_left_up )
        wx.EVT_ENTER_WINDOW( self, self._on_enter )
        wx.EVT_LEAVE_WINDOW( self, self._on_leave )

    #---------------------------------------------------------------------------
    #  Gets/Sets the current selection state of the image:
    #---------------------------------------------------------------------------

    def Selected ( self, selected = None ):
        """ Gets or sets the selection state of the image.
        """
        if selected is not None:
            selected = (selected != 0)
            if selected != self._selected:
                if selected:
                    for control in self.GetParent().GetChildren():
                        if (isinstance( control, ImageControl ) and
                            control.Selected()):
                            control.Selected( False )
                            break

                self._selected = selected
                self.Refresh()

        return self._selected

    #---------------------------------------------------------------------------
    #  Gets/Sets the current bitmap image:
    #---------------------------------------------------------------------------

    def Bitmap ( self, bitmap = None ):
        """ Gets or sets the bitmap image.
        """
        if bitmap is not None:
            if bitmap != self._bitmap:
                self._bitmap = bitmap
                self.Refresh()

        return self._bitmap

    #---------------------------------------------------------------------------
    #  Gets/Sets the current click handler:
    #---------------------------------------------------------------------------

    def Handler ( self, handler = None ):
        """ Gets or sets the click handler.
        """
        if handler is not None:
            if handler != self._handler:
                self._handler = handler
                self.Refresh()

        return self._handler

    #---------------------------------------------------------------------------
    #  Handles the mouse entering the control:
    #---------------------------------------------------------------------------

    def _on_enter ( self, event = None ):
        """ Handles the mouse entering the control.
        """
        if self._selected is not None:
            self._mouse_over = True
            self.Refresh()

    #---------------------------------------------------------------------------
    #  Handles the mouse leaving the control:
    #---------------------------------------------------------------------------

    def _on_leave ( self, event = None ):
        """ Handles the mouse leaving the control.
        """
        if self._mouse_over:
            self._mouse_over = False
            self.Refresh()

    #---------------------------------------------------------------------------
    #  Handles the user pressing the mouse button:
    #---------------------------------------------------------------------------

    def _on_left_down ( self, event = None ):
        """ Handles the user pressing the mouse button.
        """
        if self._selected is not None:
            self.CaptureMouse()
            self._button_down = True
            self.Refresh()

    #---------------------------------------------------------------------------
    #  Handles the user clicking the control:
    #---------------------------------------------------------------------------

    def _on_left_up ( self, event = None ):
        """ Handles the user clicking the control.
        """
        need_refresh = self._button_down
        if need_refresh:
            self.ReleaseMouse()
            self._button_down = False

        if self._selected is not None:
            wdx, wdy = self.GetClientSizeTuple()
            x        = event.GetX()
            y        = event.GetY()
            if (0 <= x < wdx) and (0 <= y < wdy):
                if self._selected != -1:
                    self.Selected( True )
                elif need_refresh:
                    self.Refresh()
                if self._handler is not None:
                    self._handler( self )
                return

        if need_refresh:
            self.Refresh()

    #---------------------------------------------------------------------------
    #  Handles the control being re-painted:
    #---------------------------------------------------------------------------

    def _on_paint ( self, event = None ):
        """ Handles the control being re-painted.
        """
        wdc      = wx.PaintDC( self )
        wdx, wdy = self.GetClientSizeTuple()
        bitmap   = self._bitmap
        bdx      = bitmap.GetWidth()
        bdy      = bitmap.GetHeight()
        wdc.DrawBitmap( bitmap, (wdx - bdx) / 2, (wdy - bdy) / 2, True )

        pens = [ self._selectedPenLight, self._selectedPenDark ]
        bd   = self._button_down

        if self._mouse_over:
            wdc.SetBrush( wx.TRANSPARENT_BRUSH )
            wdc.SetPen( pens[ bd ] )
            wdc.DrawLine( 0, 0, wdx, 0 )
            wdc.DrawLine( 0, 1, 0, wdy )
            wdc.SetPen( pens[ 1 - bd ] )
            wdc.DrawLine( wdx - 1, 1, wdx - 1, wdy )
            wdc.DrawLine( 1, wdy - 1, wdx - 1, wdy - 1 )

        if self._selected is True:
            wdc.SetBrush( wx.TRANSPARENT_BRUSH )
            wdc.SetPen( pens[ bd ] )
            wdc.DrawLine( 1, 1, wdx - 1, 1 )
            wdc.DrawLine( 1, 1, 1, wdy - 1 )
            wdc.DrawLine( 2, 2, wdx - 2, 2 )
            wdc.DrawLine( 2, 2, 2, wdy - 2 )
            wdc.SetPen( pens[ 1 - bd ] )
            wdc.DrawLine( wdx - 2, 2, wdx - 2, wdy - 1 )
            wdc.DrawLine( 2, wdy - 2, wdx - 2, wdy - 2 )
            wdc.DrawLine( wdx - 3, 3, wdx - 3, wdy - 2 )
            wdc.DrawLine( 3, wdy - 3, wdx - 3, wdy - 3 )




#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" The interface of a top-level application window. """


# Enthought library imports.
from traits.api import Instance, List

# Local imports.
from action.api import MenuBarManager, StatusBarManager, ToolBarManager
from image_resource import ImageResource
from i_window import IWindow


class IApplicationWindow(IWindow):
    """ The interface for a top-level application window.

    The application window has support for a menu bar, tool bar and a status
    bar (all of which are optional).

    Usage: Create a sub-class of this class and override the protected
    '_create_contents' method.
    """

    #### 'IApplicationWindow' interface #######################################

    # The window icon.  The default is toolkit specific.
    icon = Instance(ImageResource)

    # The menu bar manager (None iff there is no menu bar).
    menu_bar_manager = Instance(MenuBarManager)

    # The status bar manager (None iff there is no status bar).
    status_bar_manager = Instance(StatusBarManager)

    # The tool bar manager (None iff there is no tool bar).
    tool_bar_manager = Instance(ToolBarManager)

    # If the underlying toolkit supports multiple toolbars, you can use this
    # list instead of the single ToolBarManager instance above.
    tool_bar_managers = List(ToolBarManager)

    ###########################################################################
    # Protected 'IApplicationWindow' interface.
    ###########################################################################

    def _create_contents(self, parent):
        """ Create and return the window's contents.

        parent is the parent control.
        """

    def _create_menu_bar(self, parent):
        """ Creates the menu bar (if required).

        parent is the parent control.
        """

    def _create_status_bar(self, parent):
        """ Creates the status bar (if required).

        parent is the parent control.
        """

    def _create_tool_bar(self, parent):
        """ Creates the tool bar (if required).

        parent is the parent control.
        """

    def _create_trim_widgets(self, parent):
        """ Creates the 'trim' widgets (the widgets around the window).

        parent is the parent control.
        """

    def _set_window_icon(self):
        """ Sets the window icon (if required). """


class MApplicationWindow(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IApplicationWindow interface.

    Implements: destroy(), _create_trim_widgets()
    """

    ###########################################################################
    # 'IWidget' interface.
    ###########################################################################

    def destroy(self):
        """ Destroy the control if it exists. """
        
        if self.menu_bar_manager is not None:
            self.menu_bar_manager.destroy()
            
        if self.tool_bar_manager is not None:
            self.tool_bar_manager.destroy()
        for tool_bar_manager in self.tool_bar_managers:
            tool_bar_manager.destroy()

        super(MApplicationWindow, self).destroy()

    ###########################################################################
    # Protected 'IApplicationWindow' interface.
    ###########################################################################

    def _create_trim_widgets(self, parent):
        """ Creates the 'trim' widgets (the widgets around the window). """

        # All of these are optional.
        self._set_window_icon()
        self._create_menu_bar(parent)
        self._create_tool_bar(parent)
        self._create_status_bar(parent)

        return

#### EOF ######################################################################



#!/usr/bin/env python
# asm6502.py

# NOTE 11/21/15: Used by permission of David Beazley

"""
A simple but powerful 6502 assembler

Author : David Beazley (http://www.dabeaz.com)
Copyright (C) 2010

Parses assembly language of the following form:

var=value               ; Variable assignment

label:   OP ADDR        ; Labeled opcode and address
         OP ADDR        ; Opcode and address

Addressing modes are as follows:

         #value         ; immediate mode (8-bit value)
         %value         ; zero-page mode
         %value,X       ; zero-page X indexed
         %value,Y       ; zero-page Y indexed
         value          ; absolute
         value,X        ; absolute Y indexed
         value,Y        ; absolute Y indexed
         [value]        ; indirect
         [value,X]      ; indirect, X indexed
         [value,Y]      ; indirect, Y indexed

values and labels can be any Python expression, but the final value must
evaluate to an integer value.   Use a numeric label to set the memory
location of instructions to follow.
"""
from collections import Callable
import re

# Exception used for errors
class AssemblyError(Exception): pass

# Functions used in the creation of object code (used in the table below)
def VALUE_L(pc, value):
    return value & 0xff

def VALUE_H(pc, value):
    return (value & 0xff00) >> 8

def RELATIVE_ADDR(pc, value):
    offset = value - (pc + 2)
    return offset & 0xff

# Table of 6502 opcodes and supported addressing modes
opcodes_6502 = {
    'ADC' : {
        'immed' :      [0x69, VALUE_L],
        'zerop' :      [0x65, VALUE_L],
        'zerop_x' :    [0x75, VALUE_L],
        'abs' :        [0x6D, VALUE_L, VALUE_H],
        'abs_x' :      [0x7D, VALUE_L, VALUE_H],
        'abs_y' :      [0x79, VALUE_L, VALUE_H],
        'indirect_x' : [0x61, VALUE_L],
        'indirect_y' : [0x71, VALUE_L],
        },
    'AND' : {
        'immed' :      [0x29, VALUE_L],
        'zerop' :      [0x25, VALUE_L],
        'zerop_x' :    [0x35, VALUE_L],
        'abs' :        [0x2D, VALUE_L, VALUE_H],
        'abs_x' :      [0x3D, VALUE_L, VALUE_H],
        'abs_y' :      [0x39, VALUE_L, VALUE_H],
        'indirect_x' : [0x21, VALUE_L],
        'indirect_y' : [0x31, VALUE_L],
        },
    'ASL' : {
        'accum' :      [0x0a],
        'zerop' :      [0x06, VALUE_L],
        'zerop_x' :    [0x16, VALUE_L],
        'abs' :        [0x0e, VALUE_L, VALUE_H],
        'abs_x' :      [0x1e, VALUE_L, VALUE_H],
        },
    'BIT' : {
        'zerop' :      [0x24, VALUE_L],
        'abs'   :      [0x2c, VALUE_L, VALUE_H],
       },
    'BPL' : {
        'immed' :      [0x10, VALUE_L],
        'abs'   :      [0x10, RELATIVE_ADDR],
        },
    'BMI' : {
        'immed' :      [0x30, VALUE_L],
        'abs'   :      [0x30, RELATIVE_ADDR],
        },
    'BVC' : {
        'immed' :      [0x50, VALUE_L],
        'abs'   :      [0x50, RELATIVE_ADDR],
        },
    'BVS' : {
        'immed' :      [0x70, VALUE_L],
        'abs'   :      [0x70, RELATIVE_ADDR],
        },
    'BCC' : {
        'immed' :      [0x90, VALUE_L],
        'abs'   :      [0x90, RELATIVE_ADDR],
        },
    'BCS' : {
        'immed' :      [0xb0, VALUE_L],
        'abs'   :      [0xb0, RELATIVE_ADDR],
        },
    'BNE' : {
        'immed' :      [0xd0, VALUE_L],
        'abs'   :      [0xd0, RELATIVE_ADDR],
        },
    'BEQ' : {
        'immed' :      [0xf0, VALUE_L],
        'abs'   :      [0xf0, RELATIVE_ADDR],
        },
    'BRK' : {
        'accum' :      [0x00],
        'immed' :      [0x00, VALUE_L],
        },
    'CMP' : {
        'immed' :      [0xc9, VALUE_L],
        'zerop' :      [0xc5, VALUE_L],
        'zerop_x' :    [0xd5, VALUE_L],
        'abs' :        [0xcD, VALUE_L, VALUE_H],
        'abs_x' :      [0xdD, VALUE_L, VALUE_H],
        'abs_y' :      [0xd9, VALUE_L, VALUE_H],
        'indirect_x' : [0xc1, VALUE_L],
        'indirect_y' : [0xd1, VALUE_L],
        },
    'CPX' : {
        'immed' :      [0xe0, VALUE_L],
        'zerop' :      [0xe4, VALUE_L],
        'abs'   :      [0xec, VALUE_L, VALUE_H],
        },
    'CPY' : {
        'immed' :      [0xc0, VALUE_L],
        'zerop' :      [0xc4, VALUE_L],
        'abs'   :      [0xcc, VALUE_L, VALUE_H],
        },
    'DEC' : {
        'zerop' :      [0xc6, VALUE_L],
        'zerop_x' :    [0xd6, VALUE_L],
        'abs' :        [0xce, VALUE_L, VALUE_H],
        'abs_x' :      [0xde, VALUE_L, VALUE_H],
        },
    'EOR' : {
        'immed' :      [0x49, VALUE_L],
        'zerop' :      [0x45, VALUE_L],
        'zerop_x' :    [0x55, VALUE_L],
        'abs' :        [0x4D, VALUE_L, VALUE_H],
        'abs_x' :      [0x5D, VALUE_L, VALUE_H],
        'abs_y' :      [0x59, VALUE_L, VALUE_H],
        'indirect_x' : [0x41, VALUE_L],
        'indirect_y' : [0x51, VALUE_L],
        },
    'CLC' : {
        'accum' :      [0x18],
        },
    'SEC' : {
        'accum' :      [0x38],
        },
    'CLI' : {
        'accum' :      [0x58],
        },
    'SEI' : {
        'accum' :      [0x78],
        },
    'CLV' : {
        'accum' :      [0xb8],
        },
    'CLD' : {
        'accum' :      [0xd8],
        },
    'SED' : {
        'accum' :      [0xf8],
        },
    'INC' : {
        'zerop' :      [0xe6, VALUE_L],
        'zerop_x' :    [0xf6, VALUE_L],
        'abs' :        [0xee, VALUE_L, VALUE_H],
        'abs_x' :      [0xfe, VALUE_L, VALUE_H],
        },
    'JMP' : {
        'abs' :        [0x4c, VALUE_L, VALUE_H],
        'indirect' :   [0x6c, VALUE_L, VALUE_H]
        },
    'JSR' : {
        'abs' :        [0x20, VALUE_L, VALUE_H],
        },
    'LDA' : {
        'immed' :      [0xA9, VALUE_L],
        'zerop' :      [0xA5, VALUE_L],
        'zerop_x' :    [0xB5, VALUE_L],
        'abs' :        [0xAD, VALUE_L, VALUE_H],
        'abs_x' :      [0xBD, VALUE_L, VALUE_H],
        'abs_y' :      [0xB9, VALUE_L, VALUE_H],
        'indirect_x' : [0xA1, VALUE_L],
        'indirect_y' : [0xB1, VALUE_L],
        },
    'LDX' : {
        'immed' :      [0xa2, VALUE_L],
        'zerop' :      [0xa6, VALUE_L],
        'zerop_y' :    [0xb6, VALUE_L],
        'abs' :        [0xae, VALUE_L, VALUE_H],
        'abs_y' :      [0xbe, VALUE_L, VALUE_H],
        },
    'LDY' : { 
        'immed' :      [0xa0, VALUE_L],
        'zerop' :      [0xa4, VALUE_L],
        'zerop_x' :    [0xb4, VALUE_L],
        'abs' :        [0xac, VALUE_L, VALUE_H],
        'abs_x' :      [0xbc, VALUE_L, VALUE_H],
        },
    'LSR' : {
        'accum' :      [0x4a],
        'zerop' :      [0x46, VALUE_L],
        'zerop_x' :    [0x56, VALUE_L],
        'abs' :        [0x4e, VALUE_L, VALUE_H],
        'abs_x' :      [0x5e, VALUE_L, VALUE_H],
        },
    'NOP' : {
        'accum' :      [0xea],
        },
    'ORA' : {
        'immed' :      [0x09, VALUE_L],
        'zerop' :      [0x05, VALUE_L],
        'zerop_x' :    [0x15, VALUE_L],
        'abs' :        [0x0D, VALUE_L, VALUE_H],
        'abs_x' :      [0x1D, VALUE_L, VALUE_H],
        'abs_y' :      [0x19, VALUE_L, VALUE_H],
        'indirect_x' : [0x01, VALUE_L],
        'indirect_y' : [0x11, VALUE_L],
        },
    'TAX' : {
        'accum' :      [0xaa],
        },
    'TXA' : {
        'accum' :      [0x8a],
        },
    'DEX' : {
        'accum' :      [0xca],
        },
    'INX' : {
        'accum' :      [0xe8],
        },
    'TAY' : {
        'accum' :      [0xa8],
        },
    'TYA' : {
        'accum' :      [0x98],
        },
    'DEY' : {
        'accum' :      [0x88],
        },
    'INY' : {
        'accum' :      [0xc8],
        },
    'ROL' : {
        'accum' :      [0x2a],
        'zerop' :      [0x26, VALUE_L],
        'zerop_x' :    [0x36, VALUE_L],
        'abs' :        [0x2e, VALUE_L, VALUE_H],
        'abs_x' :      [0x3e, VALUE_L, VALUE_H],
        },
    'ROR' : {
        'accum' :      [0x6a],
        'zerop' :      [0x66, VALUE_L],
        'zerop_x' :    [0x76, VALUE_L],
        'abs' :        [0x6e, VALUE_L, VALUE_H],
        'abs_x' :      [0x7e, VALUE_L, VALUE_H],
        },
    'RTI' : {
        'accum' :      [0x40],
        },
    'RTS' : {
        'accum' :      [0x60],
        },
    'SBC' : {
        'immed' :      [0xe9, VALUE_L],
        'zerop' :      [0xe5, VALUE_L],
        'zerop_x' :    [0xf5, VALUE_L],
        'abs' :        [0xeD, VALUE_L, VALUE_H],
        'abs_x' :      [0xfD, VALUE_L, VALUE_H],
        'abs_y' :      [0xf9, VALUE_L, VALUE_H],
        'indirect_x' : [0xe1, VALUE_L],
        'indirect_y' : [0xf1, VALUE_L],
        },
    'STA' : {
        'zerop' :      [0x85, VALUE_L],
        'zerop_x' :    [0x95, VALUE_L],
        'abs' :        [0x8D, VALUE_L, VALUE_H],
        'abs_x' :      [0x9D, VALUE_L, VALUE_H],
        'abs_y' :      [0x99, VALUE_L, VALUE_H],
        'indirect_x' : [0x81, VALUE_L],
        'indirect_y' : [0x91, VALUE_L],
        },
    'TXS' : {
        'accum' :      [0x9a],
        },
    'TSX' : {
        'accum' :      [0xba],
        },
    'PHA' : {
        'accum' :      [0x48],
        },
    'PLA' : {
        'accum' :      [0x68],
        },
    'PHP' : {
        'accum' :      [0x08],
        },
    'PLP' : {
        'accum' :      [0x28],
        },
    'STX' : {
        'zerop' :      [0x86, VALUE_L],
        'zerop_y' :    [0x96, VALUE_L],
        'abs' :        [0x8e, VALUE_L, VALUE_H],
        },
    'STY' : {
        'zerop' :      [0x84, VALUE_L],
        'zerop_x' :    [0x94, VALUE_L],
        'abs' :        [0x8c, VALUE_L, VALUE_H],
        },
    }

# Parse address modes for various 6502 instructions
def parse_address_mode(mode):
    # Accumulator or implicit. Example: INC
    if not mode or mode == 'A':    return ("accum","0")  

    # Immediate value. Example : LDA #13  
    if mode.startswith("#"):       return ("immed", mode[1:]) 

    # Strip unneeded whitespace if not an immediate value
    mode = mode.replace(' ','')

    # Zero-page address with indexing. Example : LDA %25, X
    if mode.startswith("%"):
        if mode.endswith(",X"):    return ("zerop_x", mode[1:-2])
        elif mode.endswith(",Y"):  return ("zerop_y", mode[1:-2])
        else:                      return ("zerop", mode[1:])

    # Indirect addressing.Example : LDA [0xFF00, X]
    if mode.startswith("("):   
        if mode.endswith(",X)"):   return ("indirect_x", mode[1:-3])
        elif mode.endswith("),Y"): return ("indirect_y",mode[1:-3])
        elif mode.endswith(")"):   return ("indirect", mode[1:-1])

    # Absolute address, with indexing. Example : LDA 0xFF00, X
    if mode.endswith(",X"):        return ("abs_x",mode[:-2])
    elif mode.endswith(",Y"):      return ("abs_y",mode[:-2])
    else:                          return ("abs",mode)

# Parse an opcode line into intermediate object code. Returns a tuple
# (value, objcode) where value is a string to be evaluated in the 2nd pass
def parse_opcode(line):
    fields = line.split(None,1)
    opcode = fields[0]
    arg = fields[1] if len(fields) == 2 else ""
    mode,value = parse_address_mode(arg)
    opcodemodes = opcodes_6502.get(opcode)
    if not opcodemodes:
        raise AssemblyError("Unknown opcode '%s'" % opcode)
    objcode = opcodemodes.get(mode)
    if not objcode:
        raise AssemblyError("Invalid addressing mode '%s' for opcode %s" % (arg,opcode))
    if mode.startswith("abs"):
        zpmode = mode.replace("abs", "zerop")
        zp_objcode = opcodemodes.get(zpmode)
        if zp_objcode is not None:
            zp_objcode = list(zp_objcode)
    else:
        zp_objcode = None
    return (value, list(objcode), zp_objcode)

# Takes a sequence of lines and strip comments and blanks
def strip_lines(lines):
    for line in lines:
        comment_index = line.find(";")
        if comment_index >= 0:
            line = line[:comment_index]
        line = line.strip()
        yield line

assign_pat = re.compile(r'(\s*)([a-zA-Z_][a-zA-Z0-9_]*)(\s*=)')

# Parse lines into intermediate object code
def parse_lines(lines,symbols):
    for lineno,line in enumerate(lines,1):
        if assign_pat.match(line):
            exec(line,symbols)
        else:
            label, colon, statement = line.rpartition(":")
            try:
                yield lineno, label, parse_opcode(statement) if statement else (None,None)
            except AssemblyError as e:
                print("{0:4d} : Error : {1}".format(lineno,e))

# Assemble a sequence of lines into binary
def assemble_6502(lines,pc=0):
    objcode = []
    symbols = {}
    symbols['HIGH'] = lambda x : (x & 0xff00) >> 8
    symbols['LOW'] = lambda x : x & 0xff

    # Pass 1 : Parse instructions and create intermediate code
    for lineno, label, (value, icode) in parse_lines(lines,symbols):
        # Try to evaluate numeric labels and set the PC
        if label:
            try:
                pc = int(eval(label,symbols))
            except (ValueError,NameError):
                symbols[label] = pc

        # Store the resulting objcode for later expansion
        if icode:
            objcode.append((lineno,pc,value,icode))
            pc += len(icode)

    # Pass 2 : Create final object code by evaluating expressions
    execode = []
    for lineno, pc, value, icode in objcode:
        # Evaluate the value string
        try:
            symbols['PC'] = pc
            realvalue = eval(value,symbols)
            if isinstance(realvalue,str):
                realvalue = ord(realvalue) & 0xff
            if not isinstance(realvalue, int):
                raise TypeError("Integer expected in {0}".format(value))
        except Exception as e:
            print("{0:4d} : Error : {1}".format(lineno,e))
            realvalue = 0
        ecode = [op(pc,realvalue) if isinstance(op,Callable) else op
                 for op in icode]
        execode.append((lineno,pc,ecode))
    return execode

def assemble_text(text, pc=0):
    try:
        value, icode, zpicode = parse_opcode(text)
    except AssemblyError as e:
        raise
    symbols = {}
    symbols['HIGH'] = lambda x : (x & 0xff00) >> 8
    symbols['LOW'] = lambda x : x & 0xff
    symbols['PC'] = pc
    v = value.replace("$", "0x")
    try:
        realvalue = eval(v,symbols)
    except Exception, e:
        raise AssemblyError("Invalid number: %s" % value)
    if isinstance(realvalue,str):
        realvalue = ord(realvalue) & 0xff
    if not isinstance(realvalue, int):
        raise TypeError("Integer expected in {0}".format(value))
    if realvalue < 0x100 and zpicode is not None:
        icode = zpicode
    ecode = [op(pc,realvalue) if isinstance(op,Callable) else op
             for op in icode]
    return ecode


if __name__ == '__main__':
    import sys

    text = """lda #4
    sta 0xe40a
    lda (0x4567,X)
    lda (0x4567),Y
    lda 0x4567,Y
    lda 0x45
    lda 0x4567
    bmi 0x0040
    lda 0x4567,X
    lda 0x45,X
    lda $45,X
    lda 45,X
    rts""".upper()
    lines = strip_lines(text.splitlines())
    for line in lines:
        ecode = assemble_text(line)
        print line, [hex(a) for a in ecode]



""" A view containing an interactive Python shell. """


# Standard library imports.
import logging, sys

# Enthought library imports.
from envisage.api import IExtensionRegistry
from envisage.api import ExtensionPoint
from envisage.plugins.python_shell.api import IPythonShell
from pyface.api import PythonShell
from pyface.workbench.api import View
from traits.api import Any, Event, Instance, Property, DictStrAny, provides

# Setup a logger for this module.
logger = logging.getLogger(__name__)


class PseudoFile ( object ):
    """ Simulates a normal File object.
    """

    def __init__(self, write):
        self.write = write

    def readline(self):
        pass

    def writelines(self, l):
        map(self.write, l)

    def flush(self):
        pass

    def isatty(self):
        return 1


@provides(IPythonShell)
class PythonShellView(View):
    """ A view containing an interactive Python shell. """

    #### 'IView' interface ####################################################

    # The part's globally unique identifier.
    id = 'envisage.plugins.python_shell_view'

    # The part's name (displayed to the user).
    name = 'Python'

    # The default position of the view relative to the item specified in the
    # 'relative_to' trait.
    position = 'bottom'

    #### 'PythonShellView' interface ##########################################

    # The interpreter's namespace.
    namespace = Property(DictStrAny)

    # The names bound in the interpreter's namespace.
    names = Property

    # Original value for 'sys.stdout':
    original_stdout = Any

    # Stdout text is posted to this event
    stdout_text = Event

    #### 'IExtensionPointUser' interface ######################################

    # The extension registry that the object's extension points are stored in.
    extension_registry = Property(Instance(IExtensionRegistry))

    #### Private interface ####################################################

    # Bindings.
    _bindings = ExtensionPoint(id='envisage.plugins.python_shell.bindings')

    # Commands.
    _commands = ExtensionPoint(id='envisage.plugins.python_shell.commands')

    ###########################################################################
    # 'IExtensionPointUser' interface.
    ###########################################################################

    def _get_extension_registry(self):
        """ Trait property getter. """

        return self.window.application

    ###########################################################################
    # 'View' interface.
    ###########################################################################

    def create_control(self, parent):
        """ Creates the toolkit-specific control that represents the view. """

        self.shell = shell = PythonShell(parent)
        shell.on_trait_change(self._on_key_pressed, 'key_pressed')
        shell.on_trait_change(self._on_command_executed, 'command_executed')

        # Write application standard out to this shell instead of to DOS window
        self.on_trait_change(
            self._on_write_stdout, 'stdout_text', dispatch='ui'
        )
        self.original_stdout = sys.stdout
        sys.stdout = PseudoFile(self._write_stdout)

        # Namespace contributions.
        for bindings in self._bindings:
            for name, value in bindings.items():
                self.bind(name, value)

        for command in self._commands:
            self.execute_command(command)

        # We take note of the starting set of names and types bound in the
        # interpreter's namespace so that we can show the user what they have
        # added or removed in the namespace view.
        self._namespace_types = set((name, type(value)) for name, value in \
                                        self.namespace.items())

        # Register the view as a service.
        app = self.window.application
        self._service_id = app.register_service(IPythonShell, self)

        return self.shell.control

    def destroy_control(self):
        """ Destroys the toolkit-specific control that represents the view.

        """

        super(PythonShellView, self).destroy_control()

        # Unregister the view as a service.
        self.window.application.unregister_service(self._service_id)

        # Remove the sys.stdout handlers.
        self.on_trait_change(
            self._on_write_stdout, 'stdout_text', remove=True
        )

        # Restore the original stdout.
        sys.stdout = self.original_stdout

        return

    ###########################################################################
    # 'PythonShellView' interface.
    ###########################################################################

    #### Properties ###########################################################

    def _get_namespace(self):
        """ Property getter. """

        return self.shell.interpreter().locals

    def _get_names(self):
        """ Property getter. """

        return self.shell.interpreter().locals.keys()

    #### Methods ##############################################################

    def bind(self, name, value):
        """ Binds a name to a value in the interpreter's namespace. """

        self.shell.bind(name, value)

        return

    def execute_command(self, command, hidden=True):
        """ Execute a command in the interpreter. """

        return self.shell.execute_command(command, hidden)

    def execute_file(self, path, hidden=True):
        """ Execute a command in the interpreter. """

        return self.shell.execute_file(path, hidden)

    def lookup(self, name):
        """ Returns the value bound to a name in the interpreter's namespace.

        """

        return self.shell.interpreter().locals[name]

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _write_stdout(self, text):
        """ Handles text written to stdout. """

        self.stdout_text = text

        return

    #### Trait change handlers ################################################

    def _on_command_executed(self, shell):
        """ Dynamic trait change handler. """

        if self.control is not None:
            # Get the set of tuples of names and types in the current namespace.
            namespace_types = set((name, type(value)) for name, value in \
                                                        self.namespace.items())
            # Figure out the changes in the namespace, if any.
            added = namespace_types.difference(self._namespace_types)
            removed = self._namespace_types.difference(namespace_types)
            # Cache the new list, to use for comparison next time.
            self._namespace_types = namespace_types
            # Fire events if there are change.
            if len(added) > 0 or len(removed) > 0:
                self.trait_property_changed('namespace', {}, self.namespace)
                self.trait_property_changed('names', [], self.names)

        return

    def _on_key_pressed(self, event):
        """ Dynamic trait change handler. """

        if event.alt_down and event.key_code == 317:
            zoom = self.shell.control.GetZoom()
            if zoom != 20:
                self.shell.control.SetZoom(zoom+1)

        elif event.alt_down and event.key_code == 319:
            zoom = self.shell.control.GetZoom()
            if zoom != -10:
                self.shell.control.SetZoom(zoom-1)

        return

    def _on_write_stdout(self, text):
        """ Dynamic trait change handler. """

        self.shell.control.write(text)

        return

#### EOF ######################################################################



# Standard library imports.
import sys
import os

# Major package imports.
import wx
import numpy as np

# Enthought library imports.
from traits.api import Any, Bool, Int, Str, List, Event, Enum, Instance, File, Unicode, Property, provides
from pyface.key_pressed_event import KeyPressedEvent

# Local imports.
from omnivore.framework.editor import FrameworkEditor
from omnivore.framework.document import Document
from grid_control import HexEditControl
from omnivore.utils.file_guess import FileMetadata
import omnivore.utils.wx.fonts as fonts
import omnivore.utils.colors as colors
from omnivore.utils.dis6502 import Atari800Disassembler
from omnivore.utils.binutil import known_segment_parsers, ATRSegmentParser, XexSegmentParser, DefaultSegment, AnticFontSegment
from omnivore.utils.searchutil import known_searchers

from commands import PasteCommand


class HexEditor(FrameworkEditor):
    """ The toolkit specific implementation of a HexEditor.  See the
    IHexEditor interface for the API documentation.
    """

    #### 'IPythonEditor' interface ############################################

    obj = Instance(File)

    #### traits
    
    grid_range_selected = Bool
    
    segment_parser = Any
    
    segment_number = Int(0)
    
    ### View traits
    
    antic_font_data = Any
    
    antic_font = Any
    
    antic_font_mapping = Enum(0, 1)
    
    font_mode = Enum(2, 4, 5, 6, 7, 8, 9)
    
    map_width = Int
    
    map_zoom = Int
    
    bitmap_width = Int
    
    bitmap_zoom = Int
    
    playfield_colors = Any
    
    color_standard = Enum(0, 1)
    
    disassembler = Any
    
    segment = Any(None)
    
    highlight_color = Any((100, 200, 230))
    
    unfocused_cursor_color = Any((128, 128, 128))
    
    background_color = Any((255, 255, 255))
    
    match_background_color = Any((255, 255, 180))
    
    comment_background_color = Any((255, 180, 200))
    
    text_color = Any((0, 0, 0))
    
    empty_color = Any(None)
    
    text_font = Any(None)
    
    last_cursor_index = Int(0)
    
    last_anchor_start_index = Int(0)
    
    last_anchor_end_index = Int(0)

    #### Events ####

    changed = Event

    key_pressed = Event(KeyPressedEvent)
    
    # Class attributes (not traits)
    
    font_list = None
    
    searchers = known_searchers
    
    rect_select = False
    
    ##### Default traits
    
    def _disassembler_default(self):
        return Atari800Disassembler
    
    def _segment_default(self):
        return DefaultSegment()
    
    def _text_font_default(self):
        prefs = self.task.get_preferences()
        return prefs.text_font
    
    def _antic_font_data_default(self):
        return fonts.A8DefaultFont
    
    def _font_mode_default(self):
        return 2  # Antic mode 2, Graphics 0
    
    def _antic_font_mapping_default(self):
        return 1  # ATASCII
    
    def _map_width_default(self):
        return 8  # ATASCII
    
    def _map_zoom_default(self):
        return 2
    
    def _bitmap_width_default(self):
        return 1
    
    def _bitmap_zoom_default(self):
        return 5
    
    def _playfield_colors_default(self):
        return colors.powerup_colors()
    
    def _color_standard_default(self):
        return 0  # NTSC

    ###########################################################################
    # 'FrameworkEditor' interface.
    ###########################################################################

    def create(self, parent):
        self.control = self._create_control(parent)
        self.init_fonts(self.window.application)
        self.task.fonts_changed = self.font_list

    def init_extra_metadata(self, doc):
        """ Set up any pre-calculated segments based on the type or content of
        the just-loaded document.
        """
        e = doc.extra_metadata
        if 'colors' in e:
            self.update_colors(e['colors'])
        if 'font' in e:
            self.set_font(e['font'][0], e['font'][1])
        if 'initial segment' in e:
            self.initial_segment = e['initial segment']

    def rebuild_document_properties(self):
        self.find_segment()
    
    def copy_view_properties(self, old_editor):
        self.update_colors(old_editor.playfield_colors)
        self.set_font(old_editor.antic_font_data, old_editor.font_mode)
    
    def document_length(self):
        return len(self.segment)
    
    def process_paste_data_object(self, data_obj):
        bytes, extra = self.get_numpy_from_data_object(data_obj)
        cmd = PasteCommand(self.segment, self.anchor_start_index, self.anchor_end_index, bytes)
        self.process_command(cmd)
    
    def get_numpy_from_data_object(self, data_obj):
        # Full list of valid data formats:
        #
        # >>> import wx
        # >>> [x for x in dir(wx) if x.startswith("DF_")]
        # ['DF_BITMAP', 'DF_DIB', 'DF_DIF', 'DF_ENHMETAFILE', 'DF_FILENAME',
        # 'DF_HTML', 'DF_INVALID', 'DF_LOCALE', 'DF_MAX', 'DF_METAFILE',
        # 'DF_OEMTEXT', 'DF_PALETTE', 'DF_PENDATA', 'DF_PRIVATE', 'DF_RIFF',
        # 'DF_SYLK', 'DF_TEXT', 'DF_TIFF', 'DF_UNICODETEXT', 'DF_WAVE']
        extra = None
        if wx.DF_TEXT in data_obj.GetAllFormats():
            value = data_obj.GetText().encode('utf-8')
        elif wx.DF_UNICODETEXT in data_obj.GetAllFormats():  # for windows
            value = data_obj.GetText().encode('utf-8')
        else:
            value = data_obj.GetData()
            fmt = data_obj.GetPreferredFormat()
            if fmt.GetId() == "numpy,columns":
                r, c, value = value.split(",")
                extra = int(r), int(c)
        bytes = np.fromstring(value, dtype=np.uint8)
        return bytes, extra
    
    supported_clipboard_data_objects = [wx.CustomDataObject("numpy"), wx.CustomDataObject("numpy,columns"), wx.TextDataObject()]
    
    def create_clipboard_data_object(self):
        if self.anchor_start_index != self.anchor_end_index:
            data = self.segment[self.anchor_start_index:self.anchor_end_index]
            data_obj = wx.CustomDataObject("numpy")
            data_obj.SetData(data.tostring())
            return data_obj
        return None
    
    def set_text_font(self, font, color):
        self.text_color = color
        self.text_font = font
        prefs = self.task.get_preferences()
        prefs.text_font = font

    def update_panes(self):
        self.segment = self.document.segments[self.segment_number]
        self.set_colors()
        self.reconfigure_panes()
        self.update_segments_ui()
    
    def update_segments_ui(self):
        self.segment_list.set_segments(self.document.segments, self.segment_number)
        self.task.segments_changed = self.document.segments
        self.task.segment_selected = self.segment_number
    
    def reconfigure_panes(self):
        self.control.recalc_view()
        self.disassembly.recalc_view()
        self.bitmap.recalc_view()
        self.font_map.recalc_view()
        self.memory_map.recalc_view()
    
    def check_document_change(self):
        if self.last_cursor_index != self.cursor_index or self.last_anchor_start_index != self.anchor_start_index or self.last_anchor_end_index != self.anchor_end_index:
            self.document.change_count += 1
    
    def refresh_panes(self):
        self.check_document_change()
        self.control.refresh_view()
        self.disassembly.refresh_view()
        self.bitmap.refresh_view()
        self.font_map.refresh_view()
        self.memory_map.refresh_view()
    
    def set_colors(self):
        if self.empty_color is None:
            attr = self.control.GetDefaultAttributes()
            self.empty_color = attr.colBg.Get(False)
    
    def update_colors(self, colors):
        if len(colors) == 5:
            self.playfield_colors = colors
        else:
            self.playfield_colors = colors[4:9]
        self.set_font()
        self.reconfigure_panes()
    
    def set_color_standard(self, std):
        self.color_standard = std
        self.update_colors(self.playfield_colors)
    
    def get_color_converter(self):
        if self.color_standard == 0:
            return colors.gtia_ntsc_to_rgb
        return colors.gtia_pal_to_rgb
    
    def set_bitmap_width(self, width=None):
        if width is None:
            width = self.bitmap_width
        self.bitmap_width = width
        self.bitmap.recalc_view()
    
    def set_bitmap_zoom(self, zoom=None):
        if zoom is None:
            zoom = self.bitmap_zoom
        self.bitmap_zoom = zoom
        self.bitmap.recalc_view()
    
    def update_fonts(self):
        self.font_map.Refresh()
        pane = self.window.get_dock_pane('hex_edit.font_map')
        pane.name = self.font_map.get_font_mapping_name()
        self.window._aui_manager.Update()
        
    @classmethod
    def init_fonts(cls, application):
        if cls.font_list is None:
            try:
                cls.font_list = application.get_bson_data("font_list")
            except IOError:
                # file not found
                cls.font_list = []
            except ValueError:
                # bad JSON format
                cls.font_list = []
    
    def remember_fonts(self):
        self.window.application.save_bson_data("font_list", self.font_list)
    
    def get_antic_font(self):
        color_converter = self.get_color_converter()
        return fonts.AnticFont(self.antic_font_data, self.font_mode, self.playfield_colors, self.highlight_color, self.match_background_color, self.comment_background_color, color_converter)
    
    def set_font(self, font=None, font_mode=None):
        if font is None:
            font = self.antic_font_data
        if font_mode is None:
            font_mode = self.font_mode
        self.font_mode = font_mode
        self.antic_font_data = font
        self.antic_font = self.get_antic_font()
        self.font_map.set_font()
        self.set_font_mapping()
    
    def set_font_mapping(self, font_mapping=None):
        if font_mapping is None:
            font_mapping = self.antic_font_mapping
        self.antic_font_mapping = font_mapping
        self.font_map.set_font_mapping(self.antic_font_mapping)
        self.update_fonts()
    
    def set_map_width(self, width=None):
        if width is None:
            width = self.map_width
        self.map_width = width
        self.font_map.recalc_view()
    
    def set_map_zoom(self, zoom=None):
        if zoom is None:
            zoom = self.map_zoom
        self.map_zoom = zoom
        self.font_map.recalc_view()
    
    def load_font(self, filename):
        try:
            fh = open(filename, 'rb')
            data = fh.read() + "\0"*1024
            data = data[0:1024]
            font = {
                'name': os.path.basename(filename),
                'data': data,
                'char_w': 8,
                'char_h': 8,
                }
            self.set_font(font)
            self.font_list.append(font)
            self.remember_fonts()
            self.task.fonts_changed = self.font_list
        except:
            raise
    
    def get_font_from_selection(self):
        pass
    
    def set_disassembler(self, disassembler):
        self.disassembler = disassembler
        self.disassembly.recalc_view()
    
    def find_segment_parser(self, parsers, segment_name=None):
        self.document.parse_segments(parsers)
        self.find_segment(segment_name)
    
    def find_segment(self, segment_name=None):
        if segment_name is not None:
            index = self.document.find_segment_index_by_name(segment_name)
            if index < 0:
                index = 0
        else:
            index = 0
        self.segment_number = index
        self.segment_parser = self.document.segment_parser
        self.update_segments_ui()
        new_segment = self.document.segments[index]
        self.view_segment_set_width(new_segment)
        self.select_none(refresh=False)
    
    def set_segment_parser(self, parser):
        self.find_segment_parser([parser])
        self.update_panes()
    
    def view_segment_set_width(self, segment):
        self.map_width = 8
    
    def view_segment_number(self, number):
        doc = self.document
        num = number if number < len(doc.segments) else 0
        if num != self.segment_number:
            new_segment = doc.segments[num]
            self.adjust_selection(self.segment, new_segment)
            self.segment = new_segment
            self.segment_number = num
            self.invalidate_search()
            self.update_segments_ui()
            self.view_segment_set_width(new_segment)
            self.reconfigure_panes()
            self.task.status_bar.message = "Switched to segment %s" % str(self.segment)
    
    def invalidate_search(self):
        self.task.change_minibuffer_editor(self)
    
    def adjust_selection(self, current_segment, new_segment):
        """Adjust the selection of the current segment so that it is limited to the
        bounds of the new segment.
        
        If the current selection is entirely out of bounds of the new segment,
        all the selection indexes will be set to zero.
        """
        indexes = np.array([self.cursor_index, self.anchor_initial_start_index, self.anchor_start_index, self.anchor_initial_end_index, self.anchor_end_index], dtype=np.int64)
        
        # find byte index of view into master array
        current_offset = np.byte_bounds(current_segment.data)[0]
        new_offset = np.byte_bounds(new_segment.data)[0]
        
        delta = new_offset - current_offset
        indexes -= delta
        indexes.clip(0, len(new_segment) - 1, out=indexes)
        sel = indexes[1:5]
        same = (sel == sel[0])
        if same.all():
            indexes[1:5] = 0
        self.cursor_index, self.anchor_initial_start_index, self.anchor_start_index, self.anchor_initial_end_index, self.anchor_end_index = list(indexes)
    
    def get_segment_from_selection(self):
        data = self.segment[self.anchor_start_index:self.anchor_end_index]
        segment = DefaultSegment(self.segment.start_addr + self.anchor_start_index, data)
        return segment
    
    def add_user_segment(self, segment):
        self.document.add_user_segment(segment)
        self.update_segments_ui()
    
    def ensure_visible(self, start, end):
        self.index_clicked(start, 0, None)
    
    def update_history(self):
#        history = document.undo_stack.serialize()
#        self.window.application.save_log(str(history), "command_log", ".log")
        self.undo_history.update_history()

    def mark_index_range_changed(self, index_range):
        self.disassembly.restart_disassembly(index_range[0])
    
    def perform_idle(self):
        self.disassembly.perform_idle()

    ###########################################################################
    # Trait handlers.
    ###########################################################################


    ###########################################################################
    # Private interface.
    ###########################################################################

    def _create_control(self, parent):
        """ Creates the toolkit-specific control for the widget. """

        # Base-class constructor.
        self.control = HexEditControl(parent, self.task)
        self.antic_font = self.get_antic_font()

        ##########################################
        # Events.
        ##########################################

        # Get related controls
        self.disassembly = self.window.get_dock_pane('hex_edit.disassembly').control
        self.bitmap = self.window.get_dock_pane('hex_edit.bitmap').control
        self.font_map = self.window.get_dock_pane('hex_edit.font_map').control
        self.memory_map = self.window.get_dock_pane('hex_edit.memory_map').control
        self.segment_list = self.window.get_dock_pane('hex_edit.segments').control
        self.undo_history = self.window.get_dock_pane('hex_edit.undo').control

        # Load the editor's contents.
        self.load()

        return self.control

    #### wx event handlers ####################################################
    
    def index_clicked(self, index, bit, control):
        self.cursor_index = index
        self.check_document_change()
        if control != self.control:
            self.control.select_index(index)
        if control != self.disassembly:
            self.disassembly.select_index(index)
        if control != self.bitmap:
            self.bitmap.select_index(index)
        if control != self.font_map:
            self.font_map.select_index(index)
        if control != self.memory_map:
            self.memory_map.select_index(index)
        self.can_copy = (self.anchor_start_index != self.anchor_end_index)



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" Mix-in class for split widgets. """


# Enthought library imports.
from traits.api import Callable, Enum, Float, Interface


class ISplitWidget(Interface):
    """ Mix-in class for split widgets.

    A split widget is one that is split in two either horizontally or
    vertically.
    """

    #### 'ISplitWidget' interface #############################################

    # The direction in which the widget is split.
    #
    # Splitting vertically means there will be a left hand panel and a right
    # hand panel, splitting horizontally means there will be a top panel and
    # a bottom panel.
    direction = Enum('vertical', 'vertical', 'horizontal')

    # The ratio of the size of the left/top pane to the right/bottom pane.
    ratio = Float(0.5)

    # An optional callable that provides the left hand/top panel.
    lhs = Callable

    # An optional callable that provides the right hand/bottom panel.
    rhs = Callable

    ###########################################################################
    # Protected 'ISplitWidget' interface.
    ###########################################################################

    def _create_splitter(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

    def _create_lhs(self, parent):
        """ Creates the left hand/top panel depending on the direction. """

    def _create_rhs(self, parent):
        """ Creates the right hand/bottom panel depending on the direction. """


class MSplitWidget(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the ISplitWidget interface.
    """

#### EOF ######################################################################



import os
import types
import cStringIO as StringIO

import numpy as np

# Enthought library imports.
from traits.api import HasTraits, Trait, TraitHandler, Int, Any, List, Set, Bool, Event, Dict, Set, Unicode, Property, Str

from omnivore.utils.command import UndoStack
from omnivore.utils.file_guess import FileMetadata
from omnivore.utils.binutil import DefaultSegment, DefaultSegmentParser, InvalidSegmentParser


class TraitNumpyConverter(TraitHandler):
    """Trait validator to convert bytes to numpy array"""
    def validate(self, object, name, value):
        if type(value) is np.ndarray:
            return value
        elif type(value) is types.StringType:
            return np.fromstring(value, dtype=np.uint8)
        self.error(object, name, value)

    def info(self):
        return '**a string or numpy array**'


class Document(HasTraits):
    undo_stack = Any
    
    metadata = Any

    name = Property(Unicode, depends_on='metadata')

    uri = Property(Unicode, depends_on='metadata')
    
    document_id = Int(-1)
    
    last_task_id = Str
    
    bytes = Trait("", TraitNumpyConverter())
    
    segments = List
    
    segment_parser = Any
    
    user_segments = List
    
    extra_metadata = Dict
    
    # Trait events to provide view updating
    
    undo_stack_changed = Event
    
    byte_values_changed = Event  # but not the size of the bytes array. That's not handled yet
    
    change_count = Int()

    #### trait default values
    
    def _metadata_default(self):
        return FileMetadata(uri="")
    
    def _undo_stack_default(self):
        return UndoStack()
    
    def _bytes_default(self):
        return ""
    
    def _segments_default(self):
        return list([DefaultSegment(0, self.bytes)])

    #### trait property getters

    def _get_name(self):
        return self.metadata.name or 'Untitled'

    def _get_uri(self):
        return self.metadata.uri
    
    @property
    def menu_name(self):
        return "%s (%s)" % (self.name, self.uri)
    
    @classmethod
    def get_blank(cls):
        return cls(bytes="")
    
    def __str__(self):
        return self.metadata.uri
    
    def __len__(self):
        return np.alen(self.bytes)
    
    def __getitem__(self, val):
        return self.bytes[val]

    def to_bytes(self):
        return self.bytes.tostring()
    
    @property
    def bytestream(self):
        return StringIO.StringIO(self.bytes)

    def parse_segments(self, parser_list):
        parser_list.append(DefaultSegmentParser)
        for parser in parser_list:
            try:
                s = parser(self.bytes)
                break
            except InvalidSegmentParser:
                pass
        self.set_segments(s)
    
    def set_segments(self, parser):
        self.segment_parser = parser
        self.segments = []
        self.segments.extend(parser.segments)
        self.segments.extend(self.user_segments)
    
    def add_user_segment(self, segment):
        self.user_segments.append(segment)
        self.segments.append(segment)
    
    def find_segment_index_by_name(self, name):
        for i, s in enumerate(self.segments):
            if s.name == name:
                return i
        return -1
    
    def find_segment_in_range(self, addr):
        """Assuming segments had a start_addr param, find first segment that
        has addr as a valid address
        """
        for i, s in enumerate(self.segments):
            if addr >= s.start_addr and addr < (s.start_addr + len(s)):
                return i, s, addr - s.start_addr
        return -1, None, None
